@@ -1,13 +1,13 @@
 /*
-    Distorsion.h - Distorsion Effect
+    Distorsion.h - Distortion Effect
 
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2018, Will Godfrey
+    Copyright 2018-2019, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -22,20 +22,19 @@
 
     This file is derivative of ZynAddSubFX original code.
 
-    Modified February 2018
+    Modified March 2019
 */
 
 #ifndef DISTORSION_H
 #define DISTORSION_H
 
 #include "Misc/WaveShapeSamples.h"
-#include "Misc/MiscFuncs.h"
 #include "DSP/AnalogFilter.h"
 #include "Effects/Effect.h"
 
 class SynthEngine;
 
-class Distorsion : public Effect, private MiscFuncs, WaveShapeSamples
+class Distorsion : public Effect, WaveShapeSamples
 {
     public:
         Distorsion(bool insertion, float *efxoutl_, float *efxoutr_, SynthEngine *_synth);
@@ -48,32 +47,31 @@ class Distorsion : public Effect, private MiscFuncs, WaveShapeSamples
         void applyfilters(float *efxoutl, float *efxoutr);
 
     private:
-        // Parametrii
-        unsigned char Pvolume;       // Volumul or E/R
+        // Parameters
+        bool Pchanged;
+        unsigned char Pvolume;       // Volume or E/R
         unsigned char Pdrive;        // the input amplification
         unsigned char Plevel;        // the output amplification
-        unsigned char Ptype;         // Distorsion type
+        unsigned char Ptype;         // Distortion type
         unsigned char Pnegate;       // if the input is negated
-        unsigned char Plpf;          // lowpass filter
-        unsigned char Phpf;          // highpass filter
+        unsigned char Plpf;          // Lowpass filter
+        unsigned char Phpf;          // Highpass filter
         unsigned char Pstereo;       // 0 = mono, 1 = stereo
-        unsigned char Pprefiltering; // if you want to do the filtering before the distorsion
+        unsigned char Pprefiltering; // if you want to do the filtering before the distortion
 
         void setvolume(unsigned char Pvolume_);
         void setlpf(unsigned char Plpf_);
         void sethpf(unsigned char Phpf_);
 
-        InterpolatedParameter level;
+        synth::InterpolatedValue<float> level;
 
         // Real Parameters
         AnalogFilter *lpfl;
         AnalogFilter *lpfr;
         AnalogFilter *hpfl;
         AnalogFilter *hpfr;
-        InterpolatedParameter lpffr;
-        InterpolatedParameter hpffr;
-
-        SynthEngine *synth;
+        synth::InterpolatedValue<float> lpffr;
+        synth::InterpolatedValue<float> hpffr;
 };
 
 class Distlimit
