@@ -1,10 +1,11 @@
 /*
     MidiLearn.cpp
 
-    Copyright 2016-2020 Will Godfrey
+    Copyright 2016-2020, Will Godfrey
+    Copyright 2021, Will Godfrey, Rainer Hans Liffers
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -19,6 +20,9 @@
 
 */
 
+#define NEWML YES
+//#define NEWML_REPORT YES
+
 #include "Interface/MidiLearn.h"
 #include "Interface/InterChange.h"
 #include "Interface/RingBuffer.h"
@@ -32,15 +36,18 @@
 #include <vector>
 #include <string>
 #include <unistd.h>  // for usleep()
-//#include <iostream>
+#include <iostream>
+
+//#include <sys/time.h>
 
 using file::isRegularFile;
 using file::make_legit_filename;
-using file::make_legit_pathname;
 using file::setExtension;
 
 using func::asString;
 using func::asHexString;
+using std::cout;
+using std::endl;
 using std::to_string;
 using std::string;
 using std::vector;
@@ -71,7 +78,7 @@ MidiLearn::~MidiLearn()
 
 void MidiLearn::setTransferBlock(CommandBlock *getData)
 {
-    //cout << "MIDI Control " << (int) getData->data.control << " Part " << (int) getData->data.part << "  Kit " << (int) getData->data.kit << " Engine " << (int) getData->data.engine << "  Insert " << (int) getData->data.insert << endl;
+    //std::cout << "MIDI Control " << (int) getData->data.control << " Part " << (int) getData->data.part << "  Kit " << (int) getData->data.kit << " Engine " << (int) getData->data.engine << "  Insert " << (int) getData->data.insert << std::std::endl;
 
     memcpy(learnTransferBlock.bytes, getData->bytes, sizeof(learnTransferBlock));
     learnedName = resolveAll(synth, getData, false);
@@ -116,8 +123,10 @@ bool MidiLearn::runMidiLearn(int _value, unsigned short int CC, unsigned char ch
             else
                 value = _value / 128.999f; // convert from 14 bit
         }
-        else
+        else if (CC != MIDI::CC::keyPressureInner)
             value = float(_value);
+        else
+            value = float(_value >> 8);
 
         float minIn = foundEntry.min_in / 1.5748f;
         float maxIn = foundEntry.max_in / 1.5748f;
@@ -150,19 +159,14 @@ bool MidiLearn::runMidiLearn(int _value, unsigned short int CC, unsigned char ch
             value = minOut +((maxOut - minOut) * value / 127.0f);
         else if (minOut != 0) // it's just a shift
             value += minOut;
-
         CommandBlock putData;
-        putData.data.value.F = value;
-        putData.data.type = 0x40 | (foundEntry.data.type & 0x80);
+
+        memcpy(putData.bytes, foundEntry.frame.bytes, sizeof(CommandBlock));
+        putData.data.value = value;
+        putData.data.type = TOPLEVEL::type::Write | (foundEntry.frame.data.type & TOPLEVEL::type::Integer);
         // write command from midi with original integer / float type
         putData.data.source = TOPLEVEL::action::toAll;
-        putData.data.control = foundEntry.data.control;
-        putData.data.part = foundEntry.data.part;
-        putData.data.kit = foundEntry.data.kit;
-        putData.data.engine = foundEntry.data.engine;
-        putData.data.insert = foundEntry.data.insert;
-        putData.data.parameter = foundEntry.data.parameter;
-        putData.data.miscmsg = foundEntry.data.miscmsg;
+
         if (writeMidi(&putData, in_place))
         {
             if (firstLine && !in_place) // not in_place
@@ -201,7 +205,7 @@ bool MidiLearn::writeMidi(CommandBlock *putData, bool in_place)
         do
         {
             ++ tries;
-            ok = synth->interchange.fromMIDI->write(putData->bytes);
+            ok = synth->interchange.fromMIDI.write(putData->bytes);
             if (!ok)
                 usleep(1);
         // we can afford a short delay for buffer to clear
@@ -255,7 +259,7 @@ int MidiLearn::findEntry(list<LearnBlock> &midi_list, int lastpos, unsigned shor
             block->status = it->status;
             block->min_out = it->min_out;
             block->max_out = it->max_out;
-            block->data = it->data;
+            block->frame.data = it->frame.data;
             if ((it->status & 5) == 1) // blocked, not muted
                 return scan::listBlocked; // don't allow any more of this CC and channel;
             return newpos;
@@ -385,7 +389,7 @@ bool MidiLearn::remove(int itemNumber)
 
 void MidiLearn::generalOperations(CommandBlock *getData)
 {
-    int value = getData->data.value.F;
+    int value = getData->data.value;
     unsigned char type = getData->data.type;
     unsigned char control = getData->data.control;
 //    unsigned char part = getData->data.part;
@@ -393,6 +397,7 @@ void MidiLearn::generalOperations(CommandBlock *getData)
     unsigned char engine = getData->data.engine;
     unsigned char insert = getData->data.insert;
     unsigned char parameter = getData->data.parameter;
+    unsigned char offset = getData->data.offset;
     unsigned char par2 = getData->data.miscmsg;
 
     if (control == MIDILEARN::control::sendRefreshRequest)
@@ -405,7 +410,6 @@ void MidiLearn::generalOperations(CommandBlock *getData)
     if (control == MIDILEARN::control::clearAll)
     {
         midi_list.clear();
-        synth->setLastfileAdded(TOPLEVEL::XML::MLearn, "");
         updateGui();
         synth->getRuntime().Log("List cleared");
         return;
@@ -416,8 +420,10 @@ void MidiLearn::generalOperations(CommandBlock *getData)
     {
         name = (textMsgBuffer.fetch(par2));
         if (loadList(name))
+        {
+            updateGui();
             synth->getRuntime().Log("Loaded " + name);
-        updateGui();
+        }
         synth->getRuntime().finishedCLI = true;
         return;
     }
@@ -468,7 +474,7 @@ void MidiLearn::generalOperations(CommandBlock *getData)
     list<LearnBlock>::iterator it = midi_list.begin();
     it = midi_list.begin();
 
-    while(lineNo < value && it != midi_list.end())
+    while (lineNo < value && it != midi_list.end())
     {
         ++ it;
         ++lineNo;
@@ -598,13 +604,14 @@ void MidiLearn::generalOperations(CommandBlock *getData)
         CommandBlock putData;
         memset(&putData.bytes, 255, sizeof(putData));
         // need to work on this more
-        putData.data.value.F = value;
+        putData.data.value = value;
         putData.data.type = type;
         putData.data.control = MIDILEARN::control::ignoreMove;
         putData.data.kit = kit;
         putData.data.engine = engine;
         putData.data.insert = insert;
         putData.data.parameter = parameter;
+        putData.data.offset = offset;
         it->CC = kit;
         it->chan = engine;
         it->min_in = insert;
@@ -623,7 +630,7 @@ void MidiLearn::generalOperations(CommandBlock *getData)
         entry.status = type;
         entry.min_out = it->min_out;
         entry.max_out = it->max_out;
-        entry.data = it->data;
+        entry.frame.data = it->frame.data;
         unsigned int CC = entry.CC;
         int chan = entry.chan;
 
@@ -633,12 +640,12 @@ void MidiLearn::generalOperations(CommandBlock *getData)
         int lineNo = 0;
         if (midi_list.size() > 0)
         { // CC is priority
-            while(CC > it->CC && it != midi_list.end())
+            while (CC > it->CC && it != midi_list.end())
             { // find start of group
                 ++it;
                 ++lineNo;
             }
-            while(CC == it->CC && chan >= it->chan && it != midi_list.end())
+            while (CC == it->CC && chan >= it->chan && it != midi_list.end())
             { // insert at end of same channel
                 ++it;
                 ++lineNo;
@@ -661,18 +668,10 @@ void MidiLearn::generalOperations(CommandBlock *getData)
 string MidiLearn::findName(list<LearnBlock>::iterator it)
 {
     CommandBlock putData;
-    putData.data.value.I = 0;
+    memcpy(putData.bytes, it->frame.bytes, sizeof(CommandBlock));
+    putData.data.value = 0;
     putData.data.source = 0;
-
-    putData.data.type = it->data.type;
-    putData.data.control = it->data.control;
-    putData.data.part = it->data.part;
-    putData.data.kit = it->data.kit;
-    putData.data.engine = it->data.engine;
-    putData.data.insert = it->data.insert;
-    putData.data.parameter = it->data.parameter;
-    putData.data.offset = UNUSED;
-    string name = resolveAll(synth, &putData, false);;
+    string name = resolveAll(synth, &putData, false);
     return name;
 }
 
@@ -687,9 +686,9 @@ void MidiLearn::insertLine(unsigned short int CC, unsigned char chan)
         CommandBlock putData;
         int putSize = sizeof(putData);
         memset(&putData, 0xff, putSize);
-        putData.data.value.F = 0;
+        putData.data.value = 0;
         putData.data.source = TOPLEVEL::action::toAll;
-        putData.data.type = 0xc8;
+        putData.data.type = TOPLEVEL::type::Write | TOPLEVEL::type::Integer;
         putData.data.control = TOPLEVEL::control::textMessage;
         putData.data.part = TOPLEVEL::section::midiIn;
         putData.data.parameter = 0x80;
@@ -700,8 +699,10 @@ void MidiLearn::insertLine(unsigned short int CC, unsigned char chan)
     }
 
     unsigned char status = 0;
+    if (CC >= MIDI::CC::channelPressureInner)
+        status |= 1; // set 'block'
     if (CC >= MIDI::CC::identNRPN)
-        status |= 9; // mark as NRPN and set 'block'
+        status |= 8; // mark as NRPN
     LearnBlock entry;
     entry.chan = chan;
     entry.CC = CC;
@@ -709,36 +710,27 @@ void MidiLearn::insertLine(unsigned short int CC, unsigned char chan)
     entry.max_in = 200;
     entry.status = status;
 
-    //cout << "SEND Control " << (int) learnTransferBlock.data.control << " Part " << (int) learnTransferBlock.data.part << "  Kit " << (int) learnTransferBlock.data.kit << " Engine " << (int) learnTransferBlock.data.engine << "  Insert " << (int) learnTransferBlock.data.insert << endl;
+    //std::cout << "SEND Control " << (int) learnTransferBlock.data.control << " Part " << (int) learnTransferBlock.data.part << "  Kit " << (int) learnTransferBlock.data.kit << " Engine " << (int) learnTransferBlock.data.engine << "  Insert " << (int) learnTransferBlock.data.insert << std::std::endl;
 
-    unsigned char type = learnTransferBlock.data.type & 0x80;
-    learnTransferBlock.data.type = (type &0xf8) | 5; // min
+    unsigned char type = learnTransferBlock.data.type & TOPLEVEL::type::Integer;
+    learnTransferBlock.data.type = (type | TOPLEVEL::type::Limits | TOPLEVEL::type::Minimum);
     entry.min_out = synth->interchange.readAllData(&learnTransferBlock);
-    learnTransferBlock.data.type = (type &0xf8) | 6; // max
+    learnTransferBlock.data.type = (type | TOPLEVEL::type::Limits | TOPLEVEL::type::Maximum);
     entry.max_out = synth->interchange.readAllData(&learnTransferBlock);
 
-    // Should be a better way to do this!
-
-    entry.data.type = type;
-    entry.data.control = learnTransferBlock.data.control;
-    entry.data.part = learnTransferBlock.data.part;
-    entry.data.kit = learnTransferBlock.data.kit;
-    entry.data.engine = learnTransferBlock.data.engine;
-    entry.data.insert = learnTransferBlock.data.insert;
-    entry.data.parameter = learnTransferBlock.data.parameter;
-    entry.data.miscmsg = learnTransferBlock.data.miscmsg;
-
+    memcpy(entry.frame.bytes, learnTransferBlock.bytes, sizeof(CommandBlock));
+    entry.frame.data.type = type;
     list<LearnBlock>::iterator it;
     it = midi_list.begin();
     int lineNo = 0;
     if (midi_list.size() > 0)
     { // CC is priority
-        while(CC > it->CC && it != midi_list.end()) // CC is priority
+        while (CC > it->CC && it != midi_list.end()) // CC is priority
         { // find start of group
             ++it;
             ++lineNo;
         }
-        while(CC == it->CC && chan >= it->chan && it != midi_list.end())
+        while (CC == it->CC && chan >= it->chan && it != midi_list.end())
         { // insert at end of same channel
             ++it;
             ++lineNo;
@@ -764,6 +756,7 @@ void MidiLearn::insertLine(unsigned short int CC, unsigned char chan)
 
 void MidiLearn::writeToGui(CommandBlock *putData)
 {
+#ifdef GUI_FLTK
     if (!synth->getRuntime().showGui)
         return;
     putData->data.part = TOPLEVEL::section::midiLearn;
@@ -771,15 +764,17 @@ void MidiLearn::writeToGui(CommandBlock *putData)
     bool ok = false;
     do
     {
-        ok = synth->interchange.toGUI->write(putData->bytes);
+        ok = synth->interchange.toGUI.write(putData->bytes);
         ++tries;
         if (!ok)
-                usleep(1);
+                usleep(100);
         // we can afford a short delay for buffer to clear
     }
     while (!ok && tries < 3);
-    if(!ok)
-        synth->getRuntime().Log("toGui buffer full!", 2);
+
+    if (!ok)
+        synth->getRuntime().Log("toGui buffer full!", _SYS_::LogNotSerious | _SYS_::LogError);
+#endif // GUI_FLTK
 }
 
 
@@ -798,6 +793,11 @@ void MidiLearn::updateGui(int opp)
         putData.data.control = MIDILEARN::control::cancelLearn;
         putData.data.miscmsg = NO_MSG;
     }
+    else if (opp == MIDILEARN::control::limit)
+    {
+        putData.data.control = TOPLEVEL::control::textMessage;
+        putData.data.miscmsg = NO_MSG;
+    }
     else
     {
         putData.data.control = MIDILEARN::control::clearAll;
@@ -805,7 +805,7 @@ void MidiLearn::updateGui(int opp)
         if (opp == MIDILEARN::control::hideGUI)
             return;
     }
-    putData.data.value.F = 0;
+    putData.data.value = 0;
     writeToGui(&putData);
 
     if (opp >= MIDILEARN::control::hideGUI) // just sending back gui message
@@ -814,10 +814,14 @@ void MidiLearn::updateGui(int opp)
     int lineNo = 0;
     list<LearnBlock>::iterator it;
     it = midi_list.begin();
+/*
+    struct timeval tv1, tv2;
+    gettimeofday(&tv1, NULL);
+*/
     while (it != midi_list.end())
     {
         unsigned short int newCC = (it->CC) & MIDI::CC::maxNRPN;
-        putData.data.value.F = lineNo;
+        putData.data.value = lineNo;
         putData.data.type = it->status;
         putData.data.source = TOPLEVEL::action::toAll;
         putData.data.control = MIDILEARN::control::CCorChannel;
@@ -835,8 +839,20 @@ void MidiLearn::updateGui(int opp)
         }
         ++it;
         ++lineNo;
+        if (lineNo & 32)
+            usleep(10); // allow message list to clear a bit
     }
-    if (synth->getRuntime().showLearnedCC == true) // open the gui editing window
+/*
+    gettimeofday(&tv2, NULL);
+    if (tv1.tv_usec > tv2.tv_usec)
+    {
+        tv2.tv_sec--;
+        tv2.tv_usec += 1000000;
+    }
+    int actual = (tv2.tv_sec - tv1.tv_sec) *1000000 + (tv2.tv_usec - tv1.tv_usec);
+    std::cout << "Delay " << to_string(actual) << "uS" << std::endl;
+*/
+    if (synth->getRuntime().showLearnedCC == true && !midi_list.empty()) // open the gui editing window
     {
         putData.data.control = MIDILEARN::control::sendRefreshRequest;
         writeToGui(&putData);
@@ -844,7 +860,7 @@ void MidiLearn::updateGui(int opp)
 }
 
 
-bool MidiLearn::saveList(string name)
+bool MidiLearn::saveList(const string& name)
 {
     if (name.empty())
     {
@@ -859,8 +875,6 @@ bool MidiLearn::saveList(string name)
     }
 
     string file = setExtension(name, EXTEN::mlearn);
-    make_legit_pathname(file);
-
     synth->getRuntime().xmlType = TOPLEVEL::XML::MLearn;
     XMLwrapper *xml = new XMLwrapper(synth, true);
     if (!xml)
@@ -912,14 +926,14 @@ bool MidiLearn::insertMidiListData(XMLwrapper *xml)
             xml->addpar("Convert_Min", it->min_out);
             xml->addpar("Convert_Max", it->max_out);
             xml->beginbranch("COMMAND");
-                xml->addpar("Type", it->data.type);
-                xml->addpar("Control", it->data.control);
-                xml->addpar("Part", it->data.part);
-                xml->addpar("Kit_Item", it->data.kit);
-                xml->addpar("Engine", it->data.engine);
-                xml->addpar("Insert", it->data.insert);
-                xml->addpar("Parameter", it->data.parameter);
-                xml->addpar("Secondary_Parameter", it->data.miscmsg);
+                xml->addpar("Type", it->frame.data.type);
+                xml->addpar("Control", it->frame.data.control);
+                xml->addpar("Part", it->frame.data.part);
+                xml->addpar("Kit_Item", it->frame.data.kit);
+                xml->addpar("Engine", it->frame.data.engine);
+                xml->addpar("Insert", it->frame.data.insert);
+                xml->addpar("Parameter", it->frame.data.parameter);
+                xml->addpar("Secondary_Parameter", it->frame.data.offset);
                 xml->addparstr("Command_Name",findName(it).c_str());
                 xml->endbranch();
             xml->endbranch();
@@ -931,7 +945,7 @@ bool MidiLearn::insertMidiListData(XMLwrapper *xml)
 }
 
 
-bool MidiLearn::loadList(string name)
+bool MidiLearn::loadList(const string& name)
 {
     if (name.empty())
     {
@@ -939,7 +953,6 @@ bool MidiLearn::loadList(string name)
         return false;
     }
     string file = setExtension(name, EXTEN::mlearn);
-    make_legit_pathname(file);
     if (!isRegularFile(file))
     {
         synth->getRuntime().Log("Can't find " + file);
@@ -963,6 +976,7 @@ bool MidiLearn::loadList(string name)
 
 bool MidiLearn::extractMidiListData(bool full,  XMLwrapper *xml)
 {
+    midi_list.clear();
     if (!xml->enterbranch("MIDILEARN"))
     {
         if (full)
@@ -1009,14 +1023,46 @@ bool MidiLearn::extractMidiListData(bool full,  XMLwrapper *xml)
             entry.min_out = xml->getpar("Convert_Min", 0, -16384, 16383);
             entry.max_out = xml->getpar("Convert_Max", 0, -16384, 16383);
             xml->enterbranch("COMMAND");
-                entry.data.type = xml->getpar255("Type", 0); // ??
-                entry.data.control = xml->getpar255("Control", 0);
-                entry.data.part = xml->getpar255("Part", 0);
-                entry.data.kit = xml->getpar255("Kit_Item", 0);
-                entry.data.engine = xml->getpar255("Engine", 0);
-                entry.data.insert = xml->getpar255("Insert", 0);
-                entry.data.parameter = xml->getpar255("Parameter", 0);
-                entry.data.miscmsg = xml->getpar255("Secondary_Parameter", 0);
+                entry.frame.data.type = xml->getpar255("Type", 0); // ??
+                entry.frame.data.control = xml->getpar255("Control", 0);
+                entry.frame.data.part = xml->getpar255("Part", 0);
+                entry.frame.data.kit = xml->getpar255("Kit_Item", 0);
+                entry.frame.data.engine = xml->getpar255("Engine", 0);
+                entry.frame.data.insert = xml->getpar255("Insert", 0);
+                entry.frame.data.parameter = xml->getpar255("Parameter", 0);
+                entry.frame.data.offset = xml->getpar255("Secondary_Parameter", 0);
+#ifdef NEWML
+                CommandBlock allData;
+                string test = xml->getparstr("Command_Name");
+                TextData::encodeAll(synth, test, allData);
+#ifdef NEWML_REPORT
+                if (ID == 0)
+                    cout << endl;
+                cout << "line " << (ID + 1);
+                if (allData.data.control != entry.frame.data.control)
+                    cout << " changed control Old " << int(entry.frame.data.control) << " > New " << int(allData.data.control);
+                if (allData.data.part != entry.frame.data.part)
+                    cout << " changed part Old " << int(entry.frame.data.part) << " > New " << int(allData.data.part);
+                if (allData.data.kit != entry.frame.data.kit)
+                    cout << " changed kit Old " << int(entry.frame.data.kit) << " > New " << int(allData.data.kit);
+                if (allData.data.engine != entry.frame.data.engine)
+                    cout << " changed engine Old " << int(entry.frame.data.engine) << " > New " << int(allData.data.engine);
+                if (allData.data.insert != entry.frame.data.insert)
+                    cout << " changed insert Old " << int(entry.frame.data.insert) << " > New " << int(allData.data.insert);
+                if (allData.data.parameter != entry.frame.data.parameter)
+                    cout << " changed parameter Old " << int(entry.frame.data.parameter) << " > New " << int(allData.data.parameter);
+                if (allData.data.offset != entry.frame.data.offset)
+                    cout << " changed offset Old " << int(entry.frame.data.offset) << " > " << int(allData.data.offset);
+                cout << endl;
+#endif // NEWML_REPORT
+                entry.frame.data.control = allData.data.control;
+                entry.frame.data.part = allData.data.part;
+                entry.frame.data.kit = allData.data.kit;
+                entry.frame.data.engine = allData.data.engine;
+                entry.frame.data.insert = allData.data.insert;
+                entry.frame.data.parameter = allData.data.parameter;
+                entry.frame.data.offset = allData.data.offset;
+#endif // NEWML
                 xml->exitbranch();
             xml->exitbranch();
             entry.status = status;
