@@ -1,7 +1,7 @@
 /*
     TextLists.h
 
-    Copyright 2019, Will Godfrey
+    Copyright 2019-2021, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -56,6 +56,7 @@ namespace LEVEL{
         LFO, // amp/freq/filt
         Filter, // params only (slightly confused with env)
         Envelope, // amp/freq/filt/ (Sub only) band
+        Test, // special ops for Yoshimi-testsuite
     };
 }
 
@@ -116,7 +117,8 @@ namespace LISTS {
     list,
     bank,
     config,
-    mlearn
+    mlearn,
+    test,
     };
 }
 
@@ -126,11 +128,12 @@ static std::string basics [] = {
     "STop",          "all sound off",
     "RESet [s]",     "return to start-up conditions, 'ALL' clear MIDI-learn (if 'y')",
     "EXit [s]",      "tidy up and close Yoshimi (if 'y'), 'FOrce' instant exit regardless",
-    "RUN <s>",       "execute named command script",
-    "  WAIT <n>",    "1 to 1000 mS delay, within script only",
+    "RUN <s>",       "execute named command script forcing top level start",
+    "RUNLocal <s>",  "execute named command script from current context level",
+    "  WAIT <n>",    "1mS to 30,000mS delay, within script only",
     "..",            "step back one level",
     "/",             "step back to top level",
-    "end"
+    "@end","@end"
 };
 
 static std::string toplist [] = {
@@ -154,6 +157,7 @@ static std::string toplist [] = {
     "  MLearn ...",             "enter editor context level",
     "  Bank ...",               "enter context level",
     "  COnfig ...",             "enter context level",
+    "  TESt ...",               "launch test calculations (for developers)",
     "  YOshimi <n>",            "read current instance or change to n",
     "  MONo <s>",               "main output mono/stereo (ON = mono, {other})",
     "  SYStem effects [n]",     "enter effects context level",
@@ -161,30 +165,36 @@ static std::string toplist [] = {
     "    SEnd <n1> <n2>",       "send system effect to effect n1 at volume n2",
     "    (effect) <s>",         "the effect type",
     "    PREset <n>",           "set numbered effect preset to n",
-    "    -- ",                  "effect dependedent controls",
+    "    -- ",                  "effect dependent controls",
     "  INSert effects [n1]",    "enter effects context level",
     "    SEnd <s>/<n>",         "set where (Master, OFF or part number)",
     "    (effect) <s>",         "the effect type",
     "    PREset <n>",           "set numbered effect preset to n",
-    "    -- ",                  "effect dependedent controls",
+    "    -- ",                  "effect dependent controls",
     "  AVailable <n>",          "available parts (16, 32, 64)",
+    "  PANning <n>",            "panning type (Cut, Default, Boost)",
     "  Volume <n>",             "master volume",
     "  SHift <n>",              "master key shift semitones (0 no shift)",
+    "  BPM <n>",                "default BPM if none from MIDI",
     "  DEtune <n>",             "master fine detune",
-    "  SOlo [s] [n]",           "channel 'solo' switcher (Row, Column, Loop, Twoway, CC, {other} off)",
-    "      CC <n>",             "incoming 'solo' CC number (type must be set first)",
-    "end"
+    "  SOlo [s]",               "channel 'solo' switch type (ROw, COlumn, LOop, TWoway, CHannel {other} off)",
+    "  SOlo CC <n>",            "incoming 'solo' CC number (type must be set first)",
+    "  CLear <n>",              "restore instrument on part n to default settings",
+    "UNDo",                     "Revert last control change",
+    "REDo",                     "Revert last undo operation",
+    "@end","@end"
 };
 
 static std::string configlist [] = {
     "Oscillator <n>",      "* Add/Pad size (power 2 256-16384)",
     "BUffer <n>",          "* internal size (power 2 16-4096)",
     "PAdsynth [s]",        "interpolation type (Linear, other = cubic)",
+    "BUIldpad [s]",        "PADSynth wavetable build mode (Muted, Background, Autoapply)",
     "Virtual <n>",         "keyboard (0 = QWERTY, 1 = Dvorak, 2 = QWERTZ, 3 = AZERTY)",
     "Xml <n>",             "compression (0-9)",
     "REports [s]",         "destination (Stdout, other = console)",
     "SAved [s]",           "saved instrument type (Legacy {.xiz}, Yoshimi {.xiy}, Both)",
-    "ENGine [s]",          "enable instrument engines and types info (OFF, {other})",
+    //"ENGine [s]",          "enable instrument engines and types info (OFF, {other})",
     "EXPose <s>",          "show current context level (ON, OFF, PRompt)",
 
     "STate [s]",           "* autoload default at start (ON, {other})",
@@ -196,6 +206,7 @@ static std::string configlist [] = {
     "Keep [s]",            "include inactive data on all file saves (ON, {other})",
     "Gui [s]",             "* run with GUI (ON, OFF)",
     "Cli [s]",             "* run with CLI (ON, OFF)",
+    "IDentify",            "identify last bank entry fetched or added (ON, {other})",
     "LOCk <s1> <s2>",      "lock history of group s1 (ON, OFF)",
     "","INstrument, PAtchset, SCale, STate, VEctor, MLearn",
 
@@ -212,13 +223,12 @@ static std::string configlist [] = {
     "ROot [s]",            "root CC (Msb, Lsb, Off)",
     "BAnk [s]",            "bank CC (Msb, Lsb, Off)",
     "PRogram [s]",         "program change (ON, {other})",
-    "ACtivate [s]",        "program change activates part (ON, {other})",
-    "EXTended [n]",        "CC value for extended program change >= 128 is off",
+    "EXTended [n]",        "CC value for extended program change >= 120 is off",
     "Quiet [s]",           "ignore 'reset all controllers' (ON, {other})",
     "Nrpn [s]",            "incoming NRPN (ON, {other})",
     "Log [s]",             "incoming MIDI CCs (ON, {other})",
     "SHow [s]",            "GUI MIDI learn editor (ON, {other})",
-    "end"
+    "@end","@end"
 };
 
 static std::string banklist [] = {
@@ -232,37 +242,41 @@ static std::string banklist [] = {
 //    "Swap <n1> [n2]",            "Swap current bank with bank n1, (opt. in root n2)",
     "INstrument Rename <n> <s>", "change the name of slot n in the current bank",
     "INstrument SAve <n>",       "save current part's instrument to bank slot n",
-    "end"
+    "@end","@end"
 };
 
 static std::string partlist [] = {
     "<n>",                 "select part number",
     "<ON/OFF>",            "enables/disables the part",
     "Volume <n>",          "volume",
-    "Pan <n>",             "panning",
+    "Pan <n>",             "panning position",
     "VElocity <n>",        "velocity sensing sensitivity",
     "LEvel <n>",           "velocity sense offset level",
     "MIn <[s][n]>",        "minimum MIDI note value (Last seen or 0-127)",
     "MAx <[s][n]>",        "maximum MIDI note value (Last seen or 0-127)",
+    "FUll",                "reset to full key range",
     "POrtamento <s>",      "portamento (ON, {other})",
     "Mode <s>",            "key mode (Poly, Mono, Legato)",
     "Note <n>",            "note polyphony",
     "SHift <n>",           "key shift semitones (0 no shift)",
     "BYpass <n> <s>",      "bypass part effect number n, (ON, {other})",
     "EFfects [n]",         "enter effects context level",
-    "  Send <n1> <n2>",    "send part to system effect n1 at volume n2",
     "  (effect) <s>",      "the effect type",
     "  PREset <n>",        "set numbered effect preset to n",
-    "    -- ",             "effect dependedent controls",
-    "PRogram <s>/[s]<n>",  "loads instrument ID - CLear sets default,",
-    "","Group n selects from group list",
+    "  Send <n1> <n2>",    "send part to system effect n1 at volume n2",
+    "    -- ",             "effect dependent controls",
+    "PRogram <s>/[s]<n>",  "loads instrument ID - Group n selects from group list",
+    "LAtest",              "Show most recent bank load or save",
     "NAme <s>",            "sets the display name the part can be saved with",
     "TYPe <s>",            "sets the instrument type",
-    "COPyright <s>",       "sets the instrument coyright message",
+    "COPyright <s>",       "sets the instrument copyright message",
     "INFo <s>",            "fills the comments info entry",
     "Humanise Pitch [n]",  "adds a small random pitch change at note_on",
     "Humanise Velocity [n]",  "adds a small random velocity change at note_on",
-    "Channel <n>",         "MIDI channel (> 32 disables, > 16 note off only)",
+    "CLear",               "Sets current instrument level parameters to default.",
+    "CHannel <n>",         "MIDI channel (> 32 disables, > 16 note off only)",
+    "AFtertouch Chan <s1> [s2]", "Off, Filter (Down) + Peak (Down) + Bend (Down) + Modulation + Volume",
+    "AFtertouch Key <s1> [s2]",  "Off, Filter (Down) + Peak (Down) + Bend (Down) + Modulation",
     "Destination <s>",     "jack audio destination (Main, Part, Both)",
     "kit mode entries","",
     "KIT",                 "access controls but don't change type",
@@ -274,15 +288,16 @@ static std::string partlist [] = {
     "   QUiet <s>",        "silence this item (OFF, {other})",
     "   MIn <[s][n]>",     "minimum MIDI note value for this item (Last seen or 0-127)",
     "   MAx <[s][n]>",     "maximum MIDI note value for this item (Last seen or 0-127)",
+    "   FUll",             "reset to full key range",
     "   EFfect <n>",       "select effect for this item (0-none, 1-3)",
     "   NAme <s>",         "set the name for this item",
     "   DRum <s>",         "set kit to drum mode (OFF, {other})",
-    "   DIsable",          "disable kit mode",
+    "   NORmal",           "disable kit mode",
     "ADDsynth ...",        "enter AddSynth context",
     "SUBsynth ...",        "enter SubSynth context",
     "PADsynth ...",        "enter PadSynth context",
     "MCOntrol ...",        "enter MIDI controllers context",
-    "end"
+    "@end","@end"
 };
 
 static std::string mcontrollist [] = {
@@ -318,14 +333,14 @@ static std::string mcontrollist [] = {
     "E Cutoff <n>",             "emulate filter cutoff controller",
     "E Q <n>",                  "emulate filter Q controller",
     "E BAndwidth <n>",          "emulate bandwidth controller",
-    "end"
+    "@end","@end"
 };
 
 static std::string commonlist [] = {
     "ON @",                     "enables the part/kit item/engine/insert etc,",
     "OFF @",                    "disables as above",
     "Volume <n> @",             "volume",
-    "Pan <n> @",                "panning",
+    "Pan <n> @",                "panning position",
     "VElocity <n> @",           "velocity sensing sensitivity",
     "MIn <[s][n]> +",           "minimum MIDI note value (Last seen or 0-127)",
     "MAx <[s][n]> +",           "maximum MIDI note value (Last seen or 0-127)",
@@ -360,13 +375,15 @@ static std::string commonlist [] = {
     "*-voice",                  "not AddVoice",
     "&",                        "AddSynth & PadSynth only",
     "#",                        "SubSynth & PadSynth only",
-    "end"
+    "@end","@end"
 };
 
 static std::string addsynthlist [] = {
     "<ON/OFF>",                 "enables/disables the part",
     "Volume <n>",               "volume",
-    "Pan <n>",                  "panning",
+    "Pan <n>",                  "panning position",
+    "PRandom <s>",              "enable random panning(ON, {other})",
+    "PWidth <n>",               "random panning range",
     "VElocity <n>",             "velocity sensing sensitivity",
     "STEreo <s>",               "ON, {other}",
     "DEPop <n>",                "initial attack slope",
@@ -387,14 +404,16 @@ static std::string addsynthlist [] = {
     "FILter ...",               "enter Filter insert context",
     "ENVelope ...",             "enter Envelope insert context",
     "REsonance ...",            "enter Resonance context",
-    "end"
+    "@end","@end"
 };
 
 static std::string addvoicelist [] = {
     "<n>",                  "select voice number",
     "<ON/OFF>",             "enables/disables the part",
     "Volume <n>",           "volume",
-    "Pan <n>",              "panning",
+    "Pan <n>",              "panning position",
+    "PRandom <s>",          "enable random panning(ON, {other})",
+    "PWidth <n>",           "random panning range",
     "VElocity <n>",         "velocity sensing sensitivity",
     "BENd Adjust <n>",      "pitch bend range",
     "BENd Offset <n>",      "pitch bend shift",
@@ -403,15 +422,15 @@ static std::string addvoicelist [] = {
     "DETune Type <s>",      "type of coarse stepping",
     "","(DEFault, L35, L10, E100, E1200)",
     "OCTave <n>",           "shift octaves up or down",
-    "FIXed <s> *-add",      "set base frequency to 440Hz (ON, {other})",
-    "EQUal <n> *-add",      "equal temper variation",
+    "FIXed <s>",            "set base frequency to 440Hz (ON, {other})",
+    "EQUal <n>",            "equal temper variation",
     "Type <s>",             "sound type (Oscillator, White noise, Pink noise, Spot noise)",
     "SOurce <[s]/[n]>",     "voice source (Local, {voice number})",
     "OSCillator <[s]/[n]>", "oscillator source (Internal, {voice number})",
     "Phase <n>",            "relative voice phase",
     "Minus <s>",            "invert entire voice (ON, {other})",
     "DELay <n>",            "delay before this voice starts",
-    "Resonance <s>",        "enable resonance for this voice (ON, {other})",
+    "REsonance <s>",        "enable resonance for this voice (ON, {other})",
     "BYpass <s>",           "bypass global filter for this voice (ON, {other})",
     "Unison <s>",           "(ON, OFF)",
     "Unison Size <n>",      "number of unison elements",
@@ -426,37 +445,45 @@ static std::string addvoicelist [] = {
     "LFO ...",              "enter LFO insert context",
     "FILter ...",           "enter Filter insert context",
     "ENVelope ...",         "enter Envelope insert context",
-    "end"
+    "@end","@end"
 };
 
 static std::string addmodlist [] = {
-    "MOdulator",         "enter Modulator context",
+    "MOdulator",            "enter Modulator context",
     " - catgory - ","",
     "OFF",                  "disable modulator",
     "MORph","",
-    "RIng","",
-    "PHase","",
-    "FRequency","",
-    "PUlse",             "pulse width",
+    "RINg","",
+    "PHAse","",
+    "FREquency","",
+    "PULse",                "pulse width",
     "","",
-    "SOurce <[s]/[n]>",     "oscillator source (Local, {voice number})",
+//    "SIdeband <s>",         "only sidebands for ring mod (ON, {other})",
     "Volume <n>",           "volume",
     "VElocity <n>",         "velocity sensing sensitivity",
     "Damping <n>",          "higher frequency relative damping",
+    "DETune Fine <n>",      "fine frequency",
+    "DETune Coarse <n>",    "coarse stepped frequency",
+    "DETune Type <s>",      "type of coarse stepping",
+    "","(DEFault, L35, L10, E100, E1200)",
+    "SOurce <[s]/[n]>",     "oscillator source (Local, {voice number})",
     "OSCillator <[s]/[n]>", "modulation oscillator(Internal, {modulator number})",
     "FOLlow <s>",           "use source oscillator detune (ON, {other})",
+    "FIXed <s>",            "set modulator frequency to 440Hz (ON, {other})",
     "SHift <n>",            "oscillator relative phase",
     "WAveform ...",         "enter the oscillator waveform context",
-    "end"
+    "@end","@end"
 };
 
 // need to find a way to avoid this kind of duplication
-static std::string addmodnameslist [] = {"Off", "Morph", "Ring", "Phase", "Frequency", "Pulsewidth", "end"};
+static std::string addmodnameslist [] = {"Off", "Morph", "Ring", "Phase", "Frequency", "Pulsewidth", "@end"};
 
 static std::string subsynthlist [] = {
     "<ON/OFF>",                 "enables/disables the part",
     "Volume <n>",               "volume",
-    "Pan <n>",                  "panning",
+    "Pan <n>",                  "panning position",
+    "PRandom <s>",              "enable random panning(ON, {other})",
+    "PWidth <n>",               "random panning range",
     "VElocity <n>",             "velocity sensing sensitivity",
     "STEreo <s>",               "ON, {other}",
     "BENd Adjust <n>",          "pitch bend range",
@@ -466,8 +493,8 @@ static std::string subsynthlist [] = {
     "DETune Type <s>",          "type of coarse stepping",
     "","(DEFault, L35, L10, E100, E1200)",
     "OCTave <n>",               "shift octaves up or down",
-    "FIXed <s> *-add",          "set base frequency to 440Hz (ON, {other})",
-    "EQUal <n> *-add",          "equal temper variation",
+    "FIXed <s>",                "set base frequency to 440Hz (ON, {other})",
+    "EQUal <n>",                "equal temper variation",
     "OVertone Position <s>",    "relationship to fundamental",
     "","(HArmonic, SIne, POwer, SHift, UShift, LShift, UPower, LPower)",
     "OVertone First <n>",       "degree of first parameter",
@@ -483,13 +510,16 @@ static std::string subsynthlist [] = {
     "BAnd Scale <n>",           "bandwidth slope v frequency",
     "FILter ...",               "enter Filter insert context",
     "ENVelope ...",             "enter Envelope insert context",
-    "end"
+    "@end","@end"
 };
 
 static std::string padsynthlist [] = {
     "<ON/OFF>",                 "enables/disables the part",
+    "Apply",                    "use on 1st entry & harmonic changes",
     "Volume <n>",               "volume",
-    "Pan <n>",                  "panning",
+    "Pan <n>",                  "panning position",
+    "PRandom <s>",              "enable random panning(ON, {other})",
+    "PWidth <n>",               "random panning range",
     "VElocity <n>",             "velocity sensing sensitivity",
     "STEreo <s>",               "ON, {other}",
     "DEPop <n>",                "initial attack slope",
@@ -504,19 +534,19 @@ static std::string padsynthlist [] = {
     "DETune Type <s>",          "type of coarse stepping",
     "","(DEFault, L35, L10, E100, E1200)",
     "OCTave <n>",               "shift octaves up or down",
-    "FIXed <s> *-add",          "set base frequency to 440Hz (ON, {other})",
-    "EQUal <n> *-add",          "equal temper variation",
+    "FIXed <s>",                "set base frequency to 440Hz (ON, {other})",
+    "EQUal <n>",                "equal temper variation",
     "OVertone Position <s>",    "relationship to fundamental",
     "","(HArmonic, SIne, POwer, SHift, UShift, LShift, UPower, LPower)",
     "OVertone First <n>",       "degree of first parameter",
     "OVertone Second <n>",      "degree of second parameter",
     "OVertone Harmonic <n>",    "amount harmonics are forced",
     "PRofile <s>",              "shape of harmonic profile (Gauss, Square Double exponent)",
-    "WIdth <n>",                "width of harmonic profile",
+    "WIdth <n>",                "width of single peak within harmonic profile",
     "COunt <n>",                "number of profile repetitions",
-    "EXpand <n>",               "adds harmonics and changes distribution",
-    "FRequency <n>",            "further modifies distribution (dependent on stretch)",
-    "SIze <n>",                 "change harmonic width retaining shape",
+    "EXpand <n>",               "stretch and add harmonics and change distribution",
+    "FRequency <n>",            "further modifies distribution (dependent on expand)",
+    "SIze <n>",                 "scale harmonic profile as a whole",
 
     "CRoss <s>",                "cross section of profile (Full, Upper, Lower)",
     "MUltiplier <s>",           "amplitude multiplier (OFF, Gauss, Sine, Flat)",
@@ -534,6 +564,14 @@ static std::string padsynthlist [] = {
     "BAndwidth <n>",            "overall bandwidth",
     "SCale <s>",                "bandwidth scale (Normal, Equalhz, Quarter, Half, Threequart, Oneandhalf, Double, Inversehalf)",
     "SPectrum <s>",             "spectrum mode (Bandwidth, Discrete, Continuous)",
+    "XFadeupdate <n>",          "cross fade (millisec) after building new wavetable",
+
+    "BUildtrigger <n>",         "re-trigger wavetable build after n millisec",
+    "RWDetune <n>",             "random walk spread of voice detune on re-triggered build (0:off 96 factor 2)",
+    "RWBandwidth <n>",          "random walk spread of line bandwidth",
+    "RWFilterFreq <n>",         "random walk spread of filter cutoff frequency",
+    "RWWidthProfile <n>",       "random walk spread of profile line width",
+    "RWStretchProfile <n>",     "random walk spread of profile modulation stretch",
 
     "APply",                    "puts latest changes into the wavetable",
     "XPort <s>",                "export current sample set to named file",
@@ -542,7 +580,7 @@ static std::string padsynthlist [] = {
     "LFO ...",                  "enter LFO insert context",
     "FILter ...",               "enter Filter insert context",
     "ENVelope ...",             "enter Envelope insert context",
-    "end"
+    "@end","@end"
 };
 
 static std::string  resonancelist [] = {
@@ -556,7 +594,8 @@ static std::string  resonancelist [] = {
     "Smooth",              "reduce range and sharpness of peaks",
     "CLear",               "set all points to mid level",
     "POints [<n1> [n2]]",  "show all or set/read n1 to n2",
-    "end"
+    "APply",               "puts latest changes into the wavetable",
+    "@end","@end"
 };
 
 static std::string waveformlist [] = {
@@ -568,7 +607,7 @@ static std::string waveformlist [] = {
     "GAUss","",
     "DIOde","",
     "ABSsine","",
-    "PULsesine","",
+    "PSIne","",
     "STRetchsine","",
     "CHIrp","",
     "ASIne","",
@@ -576,6 +615,7 @@ static std::string waveformlist [] = {
     "SQUare","",
     "SPIke","",
     "CIRcle","",
+    "HYPersec","",
     "Harmonic <n1> Amp <n2>",   "harmonic n1 to n2 intensity",
     "Harmonic <n1> Phase <n2>", "harmonic n1 to n2 phase",
     "Harmonic Shift <n>",       "amount harmonics are moved",
@@ -602,31 +642,40 @@ static std::string waveformlist [] = {
     "ADdaptive Level <n>",      "adaptive power",
     "ADdaptive Par <n>",        "adaptive parameter",
     "APply",                    "fix settings (only for PadSynth)",
-    "end"
+    "@end","@end"
 };
 
 static std::string LFOlist [] = {
     " - catgory -","",
-    "AMplitude",          "amplitude type",
-    "FRequency",          "frequency type",
-    "FIlter",             "filter type",
+    "AMplitude",             "amplitude type",
+    "FRequency",             "frequency type",
+    "FIlter",                "filter type",
     " - control -","",
-    "Rate <n>",         "frequency",
-    "Intensity <n>",    "depth",
-    "Start <n>",        "start position in cycle",
-    "Delay <n>",        "time before effect",
-    "Expand <n>",       "rate / note pitch",
-    "Continuous <s>",   "(ON, {other})",
-    "AR <n>",           "amplitude randomness",
-    "FR <n>",           "frequency randomness",
-    "Type <s>",         "LFO oscillator shape",
-    "","SIne, TRiangle, SQuare, RUp (ramp up), RDown (ramp down), E1down, E2down",
-    "e.g. S FI T RU",     "set filter type ramp up",
-    "end"
+    "Rate [<n>][<n1> <n2>]", "frequency or BPM [1 16] to [16 1], [2 3], [3 2]",
+    "Intensity <n>",         "depth",
+    "Start <n>",             "start position in cycle",
+    "Delay <n>",             "time before effect",
+    "Expand <n>",            "rate / note pitch",
+    "Bpm <s>",               "sync frequency to MIDI clock (ON, {other})",
+    "Continuous <s>",        "(ON, {other})",
+    "AR <n>",                "amplitude randomness",
+    "RR <n>",                "frequency randomness",
+    "Type <s>",              "LFO oscillator shape",
+    "","SIne, TRiangle, SQuare, RUp (ramp up), RDown (ramp down), E1down (exp. 1), E2down (exp. 1)",
+    "","SH (sample/hold), RSU (rand square up), RSD (rand square down)",
+    "e.g. S FI T RU",        "set filter type ramp up",
+    "@end","@end"
 };
 // TODO need to find a way to safely (and efficiently) combine these
 static std::string LFOtype [] = {
-    "SIne", "TRiangle", "SQuare", "RUp (ramp up)", "RDown (ramp down)", "E1down", "E2down", "end"
+    "SIne", "TRiangle", "SQuare", "RUp (ramp up)", "RDown (ramp down)", "E1down (exp. 1)", "E2down (exp. 2)", "SH (sample/hold)", "RSU (rand square up)", "RSD (rand square dn)", "@end"
+};
+
+static std::string LFObpm [] = {
+"1/16 BPM", // space for expansion
+"1/16 BPM", "1/15 BPM", "1/14 BPM", "1/13 BPM", "1/12 BPM", "1/11 BPM", "1/10 BPM", "1/9 BPM", "1/8 BPM", "1/7 BPM", "1/6 BPM", "1/5 BPM", "1/4 BPM", "1/3 BPM", "1/2 BPM", "2/3 BPM", "1/1 BPM", "3/2 BPM", "2/1 BPM", "3/1 BPM", "4/1 BPM", "5/1 BPM", "6/1 BPM", "7/1 BPM", "8/1 BPM", "9/1 BPM", "10/1 BPM", "11/1 BPM", "12/1 BPM", "13/1 BPM", "14/1 BPM", "15/1 BPM", "16/1 BPM",
+"16/1 BPM", // space for expansion
+"Unknown BPM",
 };
 
 static std::string filterlist [] = {
@@ -672,7 +721,7 @@ static std::string filterlist [] = {
     "  FFrequency <n>", "center frequency of formant",
     "  FQ <n>",         "bandwidth of formant",
     "  FGain <n>",      "amplitude of formant",
-    "end"
+    "@end","@end"
 };
 
 static std::string envelopelist [] = {
@@ -706,7 +755,7 @@ static std::string envelopelist [] = {
     "Insert <n1> <n2> <n3>", "insert point at n1 with X increment n2, Y value n3",
     "Delete <n>",            "remove point n",
     "Change <n1> <n2> <n3>", "change point n1 to X increment n2, Y value n3",
-    "end"
+    "@end","@end"
 };
 
 static std::string reverblist [] = {
@@ -721,7 +770,7 @@ static std::string reverblist [] = {
     "TYPe <s>",         "reverb type (Random, Freeverb, Bandwidth)",
     "ROOm <n>",         "room size",
     "BANdwidth <n>",    "actual bandwidth (only for bandwidth type)",
-    "end"
+    "@end","@end"
 };
 
 static std::string echolist [] = {
@@ -732,7 +781,8 @@ static std::string echolist [] = {
     "CROssover <n>",    "left-right crossover",
     "FEEdback <n>",     "echo feedback",
     "DAMp <n>",         "feedback damping",
-    "end"
+    "BPM <s>",          "delay BPM sync (ON {other})",
+    "@end","@end"
 };
 
 static std::string choruslist [] = {
@@ -747,7 +797,9 @@ static std::string choruslist [] = {
     "FEEdback <n>",     "chorus feedback",
     "CROssover <n>",    "L/R routing",
     "SUBtract <s>",     "invert output (ON {other})",
-    "end"
+    "BPM <s>",          "LFO BPM sync (ON {other})",
+    "STArt <n>",        "LFO BPM phase start",
+    "@end","@end"
 };
 
 static std::string phaserlist [] = {
@@ -763,10 +815,12 @@ static std::string phaserlist [] = {
     "CROssover <n>",    "L/R routing",
     "SUBtract <s>",     "invert output (ON {other})",
     "RELative <n>",     "relative phase",
-    "HYPer <s>",        "hyper ?  (ON {other})",
+    "HYPer <s>",        "hyper sine (ON {other})",
     "OVErdrive <n>",    "distortion",
     "ANAlog <s>",       "analog emulation (ON {other})",
-    "end"
+    "BPM <s>",          "LFO BPM sync (ON {other})",
+    "STArt <n>",        "LFO BPM phase start",
+    "@end","@end"
 };
 
 static std::string alienwahlist [] = {
@@ -781,7 +835,9 @@ static std::string alienwahlist [] = {
     "DELay <n>",        "LFO delay",
     "CROssover <n>",    "L/R routing",
     "RELative <n>",     "relative phase",
-    "end"
+    "BPM <s>",          "LFO BPM sync (ON {other})",
+    "STArt <n>",        "LFO BPM phase start",
+    "@end","@end"
 };
 
 static std::string distortionlist [] = {
@@ -791,25 +847,25 @@ static std::string distortionlist [] = {
     "DRIve <n>",        "input level",
     "OUTput <n>",       "output balance",
     "WAVe <s>",         "function waveshape",
-    "","(ATAn, ASYm1, POWer, SINe, QNTs, ZIGzag, LMT, ULMt, LLMt, ILMt, CLIp, AS2, PO2, SGM)",
-    "INVert <s>",       "invert ?  (ON {other})",
+    "-","(ATAn, ASYm1, POWer, SINe, QNTs, ZIGzag, LMT, ULMt, LLMt, ILMt, CLIp, AS2, PO2, SGM)",
+    "INVert <s>",       "subtracts from the main signal instead of adding  (ON {other})",
     "LOW <n>",          "low pass filter",
     "HIGh <n>",         "high pass filter",
     "STEreo <s>",       "stereo (ON {other})",
     "FILter <s>",       "filter before distortion",
-    "end"
+    "@end","@end"
 };
 
 static std::string eqlist [] = {
     "LEVel <n>",        "intensity",
     "BANd <n>",         "EQ band number for following controls",
     "FILter <s>",       "filter type",
-    "","(LP1, HP1, LP2, HP2, NOT, PEA, LOW, HIG)",
+    "-","(LP1, HP1, LP2, HP2, NOT, PEA, LOW, HIG)",
     "FREquency <n>",    "cutoff/band frequency",
     "GAIn <n>",         "makeup gain",
     "Q <n>",            "filter Q",
     "STAges <n>",       "filter stages",
-    "end"
+    "@end","@end"
 };
 
 static std::string dynfilterlist [] = {
@@ -823,11 +879,13 @@ static std::string dynfilterlist [] = {
     "SENsitivity <n>",  "amount amplitude changes filter",
     "INVert <s>",       "reverse effect of sensitivity (ON {other})",
     "RATe <n>",         "speed of filter change with amplitude",
-    "FILter ...",       "enter dynamic filter context",
-    "end"
+    "FILter ...",       "enter dynamic filter context - ? FIL for controls",
+    "BPM <s>",          "LFO BPM sync (ON {other})",
+    "STArt <n>",        "LFO BPM phase start",
+    "@end","@end"
 };
 
-static std::string filtershapes [] = {"OFF" ,"ATA", "ASY", "POW", "SIN", "QNT", "ZIG", "LMT", "ULM", "LLM", "ILM", "CLI", "CLI", "AS2", "PO2", "SGM", "end"};
+static std::string filtershapes [] = {"OFF" ,"ATA", "ASY", "POW", "SIN", "QNT", "ZIG", "LMT", "ULM", "LLM", "ILM", "CLI", "CLI", "AS2", "PO2", "SGM", "@end"};
 
 static std::string learnlist [] = {
     "<n>",          "set current line number",
@@ -835,11 +893,11 @@ static std::string learnlist [] = {
     "SEven",        "set incoming NRPNs as 7 bit (ON, {other})",
     "CC <n>",       "set incoming controller value",
     "CHan <n>",     "set incoming channel number",
-    "MIn <n>",      "set minimm percentage",
+    "MIn <n>",      "set minimum percentage",
     "MAx <n>",      "set maximum percentage",
     "LImit <s>",    "limit instead of compress (ON, {other})",
     "BLock <s>",    "inhibit others on this CC/Chan pair (ON, {other})",
-    "end"
+    "@end","@end"
 };
 
 static std::string vectlist [] = {
@@ -851,7 +909,7 @@ static std::string vectlist [] = {
     "[X/Y] Control <n1> <n2>",  "sets n2 CC to use for X or Y feature n1 (2-4)",
     "OFF",                      "disable vector for this channel",
     "Name <s>",                 "text name for this complete vector",
-    "end"
+    "@end","@end"
 };
 
 static std::string scalelist [] = {
@@ -872,7 +930,7 @@ static std::string scalelist [] = {
     "NAme <s>",           "internal name for this scale",
     "DEscription <s>",    "description of this scale",
     "CLEar",              "clear all settings and revert to standard scale",
-    "end"
+    "@end","@end"
 };
 
 static std::string noteslist [] = { // from 21
@@ -893,7 +951,7 @@ static std::string loadlist [] = {
     "Patchset <s>",     "complete set of instruments from named file",
     "MLearn <s>",       "midi learned list from named file",
     "STate <s>",        "all system settings and patch sets from named file",
-    "end"
+    "@end","@end"
 };
 
 static std::string savelist [] = {
@@ -905,7 +963,7 @@ static std::string savelist [] = {
     "MLearn <s>",       "midi learned list to named file",
     "STate <s>",        "all system settings and patch sets to named file",
     "Config",           "current configuration",
-    "end",
+    "@end","@end"
 };
 
 static std::string listlist [] = {
@@ -922,7 +980,24 @@ static std::string listlist [] = {
     "History [s]",      "recent files (Patchsets, SCales, STates, Vectors, MLearn)",
     "Effects [s]",      "effect types ('all' include preset numbers and names)",
     "PREsets",          "all the presets for the currently selected effect",
-    "end"
+    "@end","@end"
+};
+
+static std::string testlist [] = {
+    "NOte [n]",         "midi note to play for test",
+    "CHannel [n]",      "midi channel to use for the test note",
+    "VElocity [n]",     "velocity to use for note on/off",
+    "DUration [n]",     "overall duration for the test sound",
+    "HOldfraction [n]", "fraction of the duration to play sound before note off",
+    "REpetitions [n]",  "number of complete test cycles to play (minimum 1)",
+    "SCalestep [n]",    "semi-tones to move up/down when repeating a test note",
+    "AOffset [n]",      "overlapping additional note with offset n, current Holdfraction",
+    "AHold [n]",        "use n as a different Holdfraction for this additional note",
+    "SWapWave [n]",     "swap wavetable of 1st PADSynth item after offset n",
+    "BUffersize [n]",   "number of samples per Synth-call < global buffsize (=default)",
+    "TArget [s]",       "target file path to write sound data (empty: /dev/null)",
+    "EXEcute",          "actually trigger the test. Stops all other sound output.",
+    "@end","@end"
 };
 
 static std::string replies [] = {
@@ -956,29 +1031,44 @@ static std::string fx_list [] = {
     "DIstortion",
     "EQ",
     "DYnfilter",
-    "end"
+    "@end"
 };
 
 static std::string type_list [] = {
     "undefined",
     "Piano",
+    "Bells and Chimes",
     "Chromatic Percussion",
     "Organ",
     "Guitar",
     "Bass",
     "Solo Strings",
     "Ensemble",
+    "Single Voice",
+    "Choir",
     "Brass",
     "Reed",
     "Pipe",
+    "Wind (other)",
     "Lead Synth",
-    "Pad Type Synth",
+    "Pad Synth",
+    "Warm Pad",
     "Synth Effects",
     "Ethnic",
     "Percussive",
     "Sound Effects",
-    "end"
+    "@end"
 };
+const int type_offset [] = {0, 1, -3, 2, 3, 4, 5, 6, 7, -6, -2, 8, 9, 10, -5, 11, 12, -4, 13, 14, 15, 16, 255};
+/*
+ * The number of the above 2 entries must match
+ * @end and 255 are the recognised terminators
+ * The list order is the display order
+ * Only add negative numbers (for backward compatibility)
+ * On old synth versions they will resolve as 'undefined'
+ *
+ * Note: can't use -1 as ID
+ */
 
 static std::string fx_presets [] = {
     "1, off",
@@ -993,30 +1083,29 @@ static std::string fx_presets [] = {
 };
 
 // effect controls
-static std::string effreverb [] = {"LEV", "PAN", "TIM", "DEL", "FEE", "none5", "none6", "LOW", "HIG", "DAM", "TYP", "ROO", "BAN", "end"};
-static std::string effecho [] = {"LEV", "PAN", "DEL", "LRD", "CRO", "FEE", "DAM",  "end"};
-static std::string effchorus [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "DEL", "FEE", "CRO", "none11", "SUB", "end"};
-static std::string effphaser [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "FEE", "STA", "CRO", "SUB", "REL", "HYP", "OVE", "ANA", "end"};
-static std::string effalienwah [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "FEE", "DEL", "CRO", "REL", "end"};
-static std::string effdistortion [] = {"LEV", "PAN", "MIX", "DRI", "OUT", "WAV", "INV", "LOW", "HIG", "STE", "FIL", "end"};
-static std::string effdistypes [] = {"ATAn", "ASYm1", "POWer", "SINe", "QNTs", "ZIGzag", "LMT", "ULMt", "LLMt", "ILMt", "CLIp", "AS2", "PO2", "SGM", "end"};
+static std::string effreverb [] = {"LEV", "PAN", "TIM", "DEL", "FEE", "none5", "none6", "LOW", "HIG", "DAM", "TYP", "ROO", "BAN", "@end"};
+static std::string effecho [] = {"LEV", "PAN", "DEL", "LRD", "CRO", "FEE", "DAM", "none7", "none8", "none9", "none10", "none11", "none12", "none13", "none14", "none15", "none16", "BPM", "@end"};
+static std::string effchorus [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "DEL", "FEE", "CRO", "none10", "SUB", "none12", "none13", "none14", "none15", "none16", "BPM", "@end"};
+static std::string effphaser [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "FEE", "STA", "CRO", "SUB", "REL", "HYP", "OVE", "ANA", "none15", "none16", "BPM", "@end"};
+static std::string effalienwah [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "FEE", "DEL", "CRO", "REL", "none11", "none12", "none13", "none14", "none15", "none16", "BPM", "@end"};
+static std::string effdistortion [] = {"LEV", "PAN", "MIX", "DRI", "OUT", "WAV", "INV", "LOW", "HIG", "STE", "FIL", "@end"};
+static std::string effdistypes [] = {"ATAn", "ASYm1", "POWer", "SINe", "QNTs", "ZIGzag", "LMT", "ULMt", "LLMt", "ILMt", "CLIp", "AS2", "PO2", "SGM", "@end"};
 static std::string effeq [] = {"LEV", "BAN", "FIL", "FRE", "GAI", "Q", "STA"};
-static std::string eqtypes [] = {"OFF", "LP1", "HP1", "LP2", "HP2", "BP2", "NOT", "PEAk", "LOW shelf", "HIGh shelf", "end"};
-static std::string effdynamicfilter [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "SEN", "INV", "RAT", "FIL", "end"};
+static std::string eqtypes [] = {"OFF", "LP1", "HP1", "LP2", "HP2", "BP2", "NOT", "PEAk", "LOW shelf", "HIGh shelf", "@end"};
+static std::string effdynamicfilter [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "SEN", "INV", "RAT", "FIL", "none11", "none12", "none13", "none14", "none15", "none16", "BPM", "@end"};
 
 // common controls
-static std::string detuneType [] = {"DEFault", "L35", "L10", "E100", "E1200", "end"};
+static std::string detuneType [] = {"DEFault", "L35", "L10", "E100", "E1200", "@end"};
 
 // waveform controls
-static std::string waveshape [] = {"Sine", "Triangle", "Pulse", "Saw", "Power", "Gauss", "Diode", "AbsSine", "PulseSine", "StretchSine", "Chirp", "AbsStretchSine", "Chebyshev", "Square", "Spike", "Circle"};
-static std::string wavebase [] = {"SIN", "TRI", "PUL", "SAW", "POW", "GAU", "DIO", "ABS", "PSI", "SSI", "CHI", "ASI", "CHE", "SQU", "SPI", "CIR", "end"};
+static std::string waveshape [] = {"Sine", "Triangle", "Pulse", "Saw", "Power", "Gauss", "Diode", "AbsSine", "PulseSine", "StretchSine", "Chirp", "AbsStretchSine", "Chebyshev", "Square", "Spike", "Circle", "HyperSec"};
+static std::string wavebase [] = {"SIN", "TRI", "PUL", "SAW", "POW", "GAU", "DIO", "ABS", "PSI", "SSI", "CHI", "ASI", "CHE", "SQU", "SPI", "CIR", "HYP", "@end"};
 static std::string basetypes [] = {"c2", "g2", "c3", "g3", "c4", "g4", "c5", "g5", "g6"};
-static std::string filtertype [] = {"OFF", "LP1", "HPA", "HPB", "BP1", "BS1", "LP2", "HP2", "BP2", "BS2", "COS", "SIN", "LSH", "SGM", "end"};
+static std::string filtertype [] = {"OFF", "LP1", "HPA", "HPB", "BP1", "BS1", "LP2", "HP2", "BP2", "BS2", "COS", "SIN", "LSH", "SGM", "@end"};
 static std::string adaptive [] = {"OFF", "ON", "SQU", "2XS", "2XA", "3XS", "3XA", "4XS", "4XA"};
 
 // misc controls
 static std::string historyGroup [] = {"IN", "PA", "SC", "ST", "VE", "ML"};
-static std::string instrumentGroupType [] = {"UNdefined", "PIano", "CHromatic percussion", "ORgan", "GUitar", "BAss", "SIngle strings", "ENsemble", "BRass", "REed", "ACcustic pipe", "LEad synth", "PAd synth", "SYnth effects", "EThnic", "PErcussive", "SOund effects"};
 
 static std::string subMagType [] = {"Linear", "40dB", "60dB", "80dB", "100dB"};
 static std::string subPadPosition [] = {"harmonic", "ushift", "lshift", "upower", "lpower", "sine", "power", "shift"};
