@@ -4,10 +4,10 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2020, Will Godfrey & others
+    Copyright 2014-2021, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -31,6 +31,7 @@
 #include <deque>
 #include <list>
 
+#include "Misc/Alloc.h"
 #include "MusicIO/MusicClient.h"
 #ifdef GUI_FLTK
 #include "FL/Fl.H"
@@ -38,6 +39,7 @@
 #include "globals.h"
 
 using std::string;
+using std::list;
 
 class XMLwrapper;
 class SynthEngine;
@@ -45,14 +47,20 @@ class SynthEngine;
 class Config
 {
     public:
-        Config(SynthEngine *_synth, int argc, char **argv);
-        ~Config();
-        bool Setup(int argc, char **argv);
-        void StartupReport(string clientName);
+        Config(SynthEngine *_synth, list<string>& allArgs, bool isLV2Plugin);
+       ~Config();
+        // shall not be copied or moved or assigned
+        Config(Config&&)                 = delete;
+        Config(Config const&)            = delete;
+        Config& operator=(Config&&)      = delete;
+        Config& operator=(Config const&) = delete;
+
+        bool Setup(void);
+        void StartupReport(const string& clientName);
         void Announce(void);
         void Usage(void);
-        void Log(const string &msg, char tostderr = 0); // 1 = cli only ored 2 = hideable
-	void LogError(const string &msg);
+        void Log(const string& msg, char tostderr = _SYS_::LogNormal);
+    void LogError(const string& msg);
         void flushLog(void);
 
         void clearPresetsDirlist(void);
@@ -63,7 +71,7 @@ class Config
         bool saveSessionData(string savefile);
         bool restoreSessionData(string sessionfile);
         bool restoreJsession();
-        void setJackSessionSave(int event_type, string session_file);
+        void setJackSessionSave(int event_type, const string& session_file);
 
         string testCCvalue(int cc);
         string masterCCtest(int cc);
@@ -74,19 +82,17 @@ class Config
         void signalCheck(void);
         void setRtprio(int prio);
         bool startThread(pthread_t *pth, void *(*thread_fn)(void*), void *arg,
-                         bool schedfifo, char lowprio, string name = "");
-        string programCmd(void) { return programcommand; }
+                         bool schedfifo, char lowprio, const string& name = "");
+        const string& programCmd(void) { return programcommand; }
 
         bool isRuntimeSetupCompleted() {return bRuntimeSetupCompleted;}
 
-        string        userHome;
-        string        ConfigDir;
-        string        localDir;
         string        defaultStateName;
         string        defaultSession;
         string        ConfigFile;
         string        paramsLoad;
         string        instrumentLoad;
+        int           load2part;
         string        midiLearnLoad;
         string        rootDefine;
         bool          stateChanged;
@@ -130,14 +136,14 @@ class Config
         int           sessionStage;
         int           Interpolation;
         string        presetsDirlist[MAX_PRESETS];
-        std::list<string> lastfileseen;
-        bool          sessionSeen[TOPLEVEL::XML::MLearn + 1];
-        bool          historyLock[TOPLEVEL::XML::MLearn + 1];
-        bool          checksynthengines;
+        list<string> lastfileseen;
+        bool          sessionSeen[TOPLEVEL::XML::ScalaMap + 1];
+        bool          historyLock[TOPLEVEL::XML::ScalaMap + 1];
         int           xmlType;
         unsigned char instrumentFormat;
         int           EnableProgChange;
         bool          toConsole;
+        int           consoleTextSize;
         bool          hideErrors;
         bool          showTimes;
         bool          logXMLheaders;
@@ -155,13 +161,18 @@ class Config
         bool          showCli;
         bool          cliChanged;
         bool          singlePath;
+        bool          banksChecked;
+        unsigned char panLaw;
         bool          configChanged;
 
+        unsigned char handlePadSynthBuild;
+        bool useLegacyPadBuild() { return handlePadSynthBuild == 0; }
+        bool usePadAutoApply()   { return handlePadSynthBuild == 2; }
+
         int           rtprio;
         int           midi_bank_root;
         int           midi_bank_C;
         int           midi_upper_voice_C;
-        int           enable_part_on_voice_load;
         bool          enable_NRPN;
         bool          ignoreResetCCs;
         bool          monitorCCin;
@@ -171,6 +182,8 @@ class Config
         int           currentPart;
         unsigned int  currentBank;
         unsigned int  currentRoot;
+        bool          bankHighlight;
+        int           lastBankPart;
         int           currentPreset;
         int           tempBank;
         int           tempRoot;
@@ -206,24 +219,24 @@ class Config
             bool Enabled[NUM_MIDI_CHANNELS];
         }vectordata;
 
-        std::list<string> LogList;
+        list<string> LogList;
 
         /*
          * These replace local memory allocations that
          * were being made every time an add or sub note
          * was processed. Now global so treat with care!
          */
-        float *genTmp1;
-        float *genTmp2;
-        float *genTmp3;
-        float *genTmp4;
+        Samples genTmp1;
+        Samples genTmp2;
+        Samples genTmp3;
+        Samples genTmp4;
 
         // as above but for part and sys effect
-        float *genMixl;
-        float *genMixr;
+        Samples genMixl;
+        Samples genMixr;
 
     private:
-        void loadCmdArgs(int argc, char **argv);
+        void applyOptions(Config*settings, list<string>& allArgs);
         void defaultPresets(void);
         bool extractBaseParameters(XMLwrapper *xml);
         bool extractConfigData(XMLwrapper *xml);
@@ -239,6 +252,7 @@ class Config
         const string programcommand;
         string jackSessionDir;
         string baseConfig;
+        string presetDir;
 
         SynthEngine *synth;
         bool bRuntimeSetupCompleted;
@@ -263,12 +277,6 @@ private:
 public:
     enum
     {
-        UpdateMaster = 0,
-        UpdateConfig,
-        UpdatePaths,
-        UpdatePart,
-        RefreshCurBank,
-        GuiCheck,
         NewSynthEngine,
         UNDEFINED = 9999
     };
