@@ -4,10 +4,10 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2020, Will Godfrey
+    Copyright 2014-2021, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -39,6 +39,7 @@
 using file::saveText;
 using file::loadGzipped;
 using file::saveGzipped;
+using file::findExtension;
 using func::string2int;
 using func::string2float;
 using func::asLongString;
@@ -89,8 +90,8 @@ XMLwrapper::XMLwrapper(SynthEngine *_synth, bool _isYoshi, bool includeBase) :
         //std::cout << "Zyn doctype" << std::endl;
         mxmlElementSetAttr(doctype, "ZynAddSubFX-data", NULL);
         root = mxmlNewElement(tree, "ZynAddSubFX-data");
-        mxmlElementSetAttr(root, "version-major", "3");
-        mxmlElementSetAttr(root, "version-minor", "0");
+        mxmlElementSetAttr(root, "version-major", "2");
+        mxmlElementSetAttr(root, "version-minor", "4");
         mxmlElementSetAttr(root, "ZynAddSubFX-author", "Nasca Octavian Paul");
         information.yoshiType = 0;
     }
@@ -112,6 +113,8 @@ XMLwrapper::XMLwrapper(SynthEngine *_synth, bool _isYoshi, bool includeBase) :
             addparbool("enable_splash", synth->getRuntime().showSplash);
             addparbool("enable_CLI", synth->getRuntime().showCli);
             addparbool("enable_single_master", synth->getRuntime().singlePath);
+            addparbool("banks_checked", synth->getRuntime().banksChecked);
+            addparU("handle_padsynth_build", synth->getRuntime().handlePadSynthBuild);
             addparbool("enable_auto_instance", synth->getRuntime().autoInstance);
             addparU("active_instances", synth->getRuntime().activeInstance);
             addpar("show_CLI_context", synth->getRuntime().showCLIcontext);
@@ -131,7 +134,7 @@ XMLwrapper::XMLwrapper(SynthEngine *_synth, bool _isYoshi, bool includeBase) :
         return;
     }
 
-    if (synth->getRuntime().xmlType <= TOPLEVEL::XML::Scale || synth->getRuntime().xmlType == TOPLEVEL::XML::Presets)
+    if (synth->getRuntime().xmlType <= TOPLEVEL::XML::Scale)
     {
             beginbranch("BASE_PARAMETERS");
                 addpar("max_midi_parts", NUM_MIDI_CHANNELS);
@@ -164,7 +167,7 @@ void XMLwrapper::checkfileinformation(const std::string& filename, unsigned int&
     std::string report = "";
     char *xmldata = loadGzipped(filename, &report);
     if (report != "")
-        synth->getRuntime().Log(report, 2);
+        synth->getRuntime().Log(report, _SYS_::LogNotSerious);
     if (!xmldata)
         return;
 
@@ -188,7 +191,7 @@ void XMLwrapper::checkfileinformation(const std::string& filename, unsigned int&
         if (idx != NULL)
         {
             seen |= 2;
-            if(strstr(idx, "name=\"ADDsynth_used\" value=\"yes\""))
+            if (strstr(idx, "name=\"ADDsynth_used\" value=\"yes\""))
                 information.ADDsynth_used = 1;
         }
 
@@ -196,7 +199,7 @@ void XMLwrapper::checkfileinformation(const std::string& filename, unsigned int&
         if (idx != NULL)
         {
             seen |= 4;
-            if(strstr(idx, "name=\"SUBsynth_used\" value=\"yes\""))
+            if (strstr(idx, "name=\"SUBsynth_used\" value=\"yes\""))
                 information.SUBsynth_used = 1;
         }
 
@@ -204,7 +207,7 @@ void XMLwrapper::checkfileinformation(const std::string& filename, unsigned int&
         if (idx != NULL)
         {
             seen |= 1;
-            if(strstr(idx, "name=\"PADsynth_used\" value=\"yes\""))
+            if (strstr(idx, "name=\"PADsynth_used\" value=\"yes\""))
                 information.PADsynth_used = 1;
         }
     }
@@ -313,7 +316,7 @@ bool XMLwrapper::saveXMLfile(std::string _filename, bool useCompression)
     {
         if (!saveText(xmldata, filename))
         {
-            synth->getRuntime().Log("XML: Failed to save xml file " + filename + " for save", 2);
+            synth->getRuntime().Log("XML: Failed to save xml file " + filename + " for save", _SYS_::LogNotSerious);
             return false;
         }
     }
@@ -324,7 +327,7 @@ bool XMLwrapper::saveXMLfile(std::string _filename, bool useCompression)
         std::string result = saveGzipped(xmldata, filename, compression);
         if (result > "")
         {
-            synth->getRuntime().Log(result,2);
+            synth->getRuntime().Log(result, _SYS_::LogNotSerious);
             return false;
         }
     }
@@ -479,17 +482,17 @@ bool XMLwrapper::loadXMLfile(const std::string& filename)
     std::string report = "";
     char *xmldata = loadGzipped(filename, &report);
     if (report != "")
-        synth->getRuntime().Log(report, 2);
+        synth->getRuntime().Log(report, _SYS_::LogNotSerious);
     if (xmldata == NULL)
     {
-        synth->getRuntime().Log("XML: Could not load xml file: " + filename, 2);
+        synth->getRuntime().Log("XML: Could not load xml file: " + filename, _SYS_::LogNotSerious);
          return false;
     }
     root = tree = mxmlLoadString(NULL, removeBlanks(xmldata), MXML_OPAQUE_CALLBACK);
     delete [] xmldata;
     if (!tree)
     {
-        synth->getRuntime().Log("XML: File " + filename + " is not XML", 2);
+        synth->getRuntime().Log("XML: File " + filename + " is not XML", _SYS_::LogNotSerious);
         return false;
     }
     root = mxmlFindElement(tree, tree, "ZynAddSubFX-data", NULL, NULL, MXML_DESCEND);
@@ -501,11 +504,12 @@ bool XMLwrapper::loadXMLfile(const std::string& filename)
 
     if (!root)
     {
-        synth->getRuntime().Log("XML: File " + filename + " doesn't contain valid data in this context", 2);
+        synth->getRuntime().Log("XML: File " + filename + " doesn't contain valid data in this context", _SYS_::LogNotSerious);
         return false;
     }
     node = root;
     push(root);
+    synth->fileCompatible = true;
     if (zynfile)
     {
         xml_version.major = string2int(mxmlElementGetAttr(root, "version-major"));
@@ -516,11 +520,15 @@ bool XMLwrapper::loadXMLfile(const std::string& filename)
         xml_version.y_major = string2int(mxmlElementGetAttr(root, "Yoshimi-major"));
         yoshitoo = true;
 
-//        synth->getRuntime().Log("XML: Yoshimi " + asString(xml_version.y_major));
+        //synth->getRuntime().Log("XML: Yoshimi " + asString(xml_version.y_major) + "  " + filename);
     }
     else
+    {
         synth->getRuntime().lastXMLmajor = 0;
-
+        if (xml_version.major > 2)
+            synth->fileCompatible = false;
+    }
+// std::cout << "major " << int(xml_version.major) << "  Yosh " << int(synth->getRuntime().lastXMLmajor) << std::endl;
     if (mxmlElementGetAttr(root, "Yoshimi-minor"))
     {
         xml_version.y_minor = string2int(mxmlElementGetAttr(root, "Yoshimi-minor"));
@@ -529,9 +537,17 @@ bool XMLwrapper::loadXMLfile(const std::string& filename)
     }
     else
         synth->getRuntime().lastXMLminor = 0;
+    string exten = findExtension(filename);
+    if (exten.length() != 4 && exten != ".state")
+        return true; // we don't want config stuff
 
     if (synth->getRuntime().logXMLheaders)
     {
+        if (yoshitoo && xml_version.major > 2)
+        { // we were giving the wrong value :(
+            xml_version.major = 2;
+            xml_version.minor = 4;
+        }
         if (zynfile)
             synth->getRuntime().Log("ZynAddSubFX version major " + asString(xml_version.major) + "   minor " + asString(xml_version.minor));
         if (yoshitoo)
@@ -753,7 +769,7 @@ void XMLwrapper::push(mxml_node_t *node)
 {
     if (stackpos >= STACKSIZE - 1)
     {
-        synth->getRuntime().Log("XML: Not good, XMLwrapper push on a full parentstack", 2);
+        synth->getRuntime().Log("XML: Not good, XMLwrapper push on a full parentstack", _SYS_::LogNotSerious);
         return;
     }
     stackpos++;
@@ -765,7 +781,7 @@ mxml_node_t *XMLwrapper::pop(void)
 {
     if (stackpos <= 0)
     {
-        synth->getRuntime().Log("XML: Not good, XMLwrapper pop on empty parentstack", 2);
+        synth->getRuntime().Log("XML: Not good, XMLwrapper pop on empty parentstack", _SYS_::LogNotSerious);
         return root;
     }
     mxml_node_t *node = parentstack[stackpos];
@@ -779,7 +795,7 @@ mxml_node_t *XMLwrapper::peek(void)
 {
     if (stackpos <= 0)
     {
-        synth->getRuntime().Log("XML: Not good, XMLwrapper peek on an empty parentstack", 2);
+        synth->getRuntime().Log("XML: Not good, XMLwrapper peek on an empty parentstack", _SYS_::LogNotSerious);
         return root;
     }
     return parentstack[stackpos];
