@@ -7,7 +7,7 @@
     Copyright 2018-2019 Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -46,12 +46,12 @@ class PresetsStore
         ~PresetsStore();
 
         // Clipboard stuff
-        void copyclipboard(XMLwrapper *xml, string type);
+        void copyclipboard(XMLwrapper *xml, const string& type);
         bool pasteclipboard(XMLwrapper *xml);
-        bool checkclipboardtype(string type);
+        bool checkclipboardtype(const string& type);
 
         // presets stuff
-        void copypreset(XMLwrapper *xml, string type, string name);
+        void copypreset(XMLwrapper *xml, const string& type, const string& name);
         bool pastepreset(XMLwrapper *xml, int npreset);
         void deletepreset(int npreset);
 
@@ -61,7 +61,7 @@ class PresetsStore
         };
         presetstruct presets[MAX_PRESETS];
 
-        void rescanforpresets(string type, int root);
+        void rescanforpresets(const string& type, int root);
 
     private:
         void clearpresets(void);
