@@ -1,10 +1,10 @@
 /*
     MiscGui.cpp - common link between GUI and synth
 
-    Copyright 2016-2019 Will Godfrey & others
+    Copyright 2016-2022 Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -21,6 +21,8 @@
 
 #include "Misc/SynthEngine.h"
 #include "Misc/TextMsgBuffer.h"
+#include "Misc/NumericFuncs.h"
+#include "Params/RandomWalk.h"
 #include "MiscGui.h"
 #include "MasterUI.h"
 
@@ -30,13 +32,17 @@
 #include <cairo.h>
 #include <cairo-xlib.h>
 
+using std::to_string;
+using std::ostringstream;
+
+using func::bpm2text;
+using func::power;
 
 namespace { // Implementation details...
 
     TextMsgBuffer& textMsgBuffer = TextMsgBuffer::instance();
 }
 
-
 float collect_readData(SynthEngine *synth, float value, unsigned char control, unsigned char part, unsigned char kititem, unsigned char engine, unsigned char insert, unsigned char parameter, unsigned char offset, unsigned char miscmsg, unsigned char request)
 {
     unsigned char type = 0;
@@ -47,7 +53,7 @@ float collect_readData(SynthEngine *synth, float value, unsigned char control, u
         action |= request;
     CommandBlock putData;
 
-    putData.data.value.F = value;
+    putData.data.value = value;
     putData.data.type = type;
     putData.data.source = action;
     putData.data.control = control;
@@ -76,13 +82,12 @@ void collect_data(SynthEngine *synth, float value, unsigned char action, unsigne
     {
         if (collect_readData(synth, 0, PART::control::partBusy, part))
         {
-            fl_alert("Part %d is busy", int(part));
+            alert(synth, "Part " + to_string(part + 1) + " is busy");
             return;
         }
     }
-
     CommandBlock putData;
-    putData.data.value.F = value;
+    putData.data.value = value;
     putData.data.control = control;
     putData.data.part = part;
     putData.data.kit = kititem;
@@ -91,83 +96,118 @@ void collect_data(SynthEngine *synth, float value, unsigned char action, unsigne
     putData.data.parameter = parameter;
     putData.data.offset = offset;
     putData.data.miscmsg = miscmsg;
+    if (action == TOPLEVEL::action::fromMIDI)
+        type = type | 1; // faking MIDI from virtual keyboard
+    else
+    {
+        if (part != TOPLEVEL::section::midiLearn)
+        { // midilearn UI must pass though un-modified
+            unsigned char typetop = type & (TOPLEVEL::type::Write | TOPLEVEL::type::Integer);
+            unsigned char buttons = Fl::event_button();
+            if (part == TOPLEVEL::section::main && (control != MAIN::control::volume &&  control  != MAIN::control::detune))
+                type = 1;
 
-    if (part != TOPLEVEL::section::midiLearn)
-    { // midilearn must pass though un-modified
-        unsigned char typetop = type & 0xc0;
-        unsigned char buttons = type & 7;
-        if (part == TOPLEVEL::section::main && (control != MAIN::control::volume &&  control  != MAIN::control::detune))
-            type = 1;
-
-        if (buttons == 3 && Fl::event_is_click())
-        {
-            float newValue;
-            putData.data.type = 3 | TOPLEVEL::type::Limits;
-            newValue = synth->interchange.readAllData(&putData);
-            //if (newValue != value)
-                //std::cout << "Gui limits " << value <<" to " << newValue << std::endl;
-            if(Fl::event_state(FL_CTRL) != 0)
+            if (buttons == 3 && Fl::event_is_click())
             {
-                if (putData.data.type & TOPLEVEL::type::Learnable)
+                // check range & if learnable
+                float newValue;
+                putData.data.type = 3 | TOPLEVEL::type::Limits;
+                newValue = synth->interchange.readAllData(&putData);
+                //if (newValue != value)
+                    //std::cout << "Gui limits " << value <<" to " << newValue << std::endl;
+                if (Fl::event_state(FL_CTRL) != 0)
                 {
-                    // identifying this for button 3 as MIDI learn
-                    type = TOPLEVEL::type::LearnRequest;
+                    if (putData.data.type & TOPLEVEL::type::Learnable)
+                    {
+                        // identifying this for button 3 as MIDI learn
+                        type = TOPLEVEL::type::LearnRequest;
+                    }
+                    else
+                    {
+                        alert(synth, "Can't learn this control");
+                        synth->getRuntime().Log("Can't MIDI-learn this control");
+                        type = TOPLEVEL::type::Learnable;
+                    }
                 }
                 else
                 {
-                    synth->getGuiMaster()->words->copy_label("Can't learn this control");
-                    synth->getGuiMaster()->message->show();
-                    synth->getGuiMaster()->message->position(Fl::event_x_root() + 16, Fl::event_y_root());
-                    synth->getRuntime().Log("Can't MIDI-learn this control");
-                    /* can't use fl_alert here.
-                     * For some reason it goes into a loop on spin boxes
-                     * and runs menus up to their max value.
-                     */
-                    type = TOPLEVEL::type::Learnable;
+                    putData.data.value = newValue;
+                    type = TOPLEVEL::type::Write;
+                    action |= TOPLEVEL::action::forceUpdate;
+                    // has to be write as it's 'set default'
                 }
             }
-            else
-            {
-                putData.data.value.F = newValue;
-                type = TOPLEVEL::type::Write;
-                action |= TOPLEVEL::action::forceUpdate;
-                // has to be write as it's 'set default'
-            }
+            else if (buttons > 2)
+                type = 1; // change scroll wheel to button 1
+            type |= typetop;
+            action |= TOPLEVEL::action::fromGUI;
         }
-        else if(buttons > 2)
-            type = 1; // change scroll wheel to button 1
-        type |= typetop;
     }
 
     putData.data.type = type;
-    putData.data.source = action | TOPLEVEL::action::fromGUI;
-//cout << "collect_data value " << value << "  action " << int(action)  << "  type " << int(type) << "  control " << int(control) << "  part " << int(part) << "  kit " << int(kititem) << "  engine " << int(engine) << "  insert " << int(insert)  << "  par " << int(parameter) << " par2 " << int(par2) << endl;
-    if (!synth->interchange.fromGUI->write(putData.bytes))
+    putData.data.source = action;
+    //cout << "collect_data value " << value << "  action " << int(action)  << "  type " << int(type) << "  control " << int(control) << "  part " << int(part) << "  kit " << int(kititem) << "  engine " << int(engine) << "  insert " << int(insert)  << "  par " << int(parameter) << " offset " << int(offset) << " msg " << int(miscmsg) << endl;
+    if (!synth->interchange.fromGUI.write(putData.bytes))
         synth->getRuntime().Log("Unable to write to fromGUI buffer.");
 }
 
+void alert(SynthEngine *synth, string message)
+{
+    synth->getGuiMaster()->query("", "", "", message);
+}
 
+int choice(SynthEngine *synth, string one, string two, string three, string message)
+{
+    return synth->getGuiMaster()->query(one, two, three, message);
+}
+
+string setfiler(SynthEngine *synth, string title, string name, bool save, int extension)
+{
+    return synth->getGuiMaster()->setfiler(title, name, save, extension);
+}
+
+string input_text(SynthEngine *synth, string label, string text)
+{
+    return synth->getGuiMaster()->setinput(label, text);
+}
+
+//static int keycheck = 0;
 void GuiUpdates::read_updates(SynthEngine *synth)
 {
     CommandBlock getData;
-    bool isChanged = false;
-    while (synth->interchange.toGUI->read(getData.bytes))
+    while (synth->interchange.toGUI.read(getData.bytes))
     {
         Fl::lock();
         decode_updates(synth, &getData);
         Fl::unlock();
-        isChanged = true;
     }
-    /*
-     * we perform all the updates that have occured in this refresh
-     * period then do just a single FLTK check for 'damage'
-     */
-    if (isChanged)
+    // and pull up to 5 entries from log
+    for (int i = 0; !synth->getRuntime().LogList.empty() && i < 5; ++i)
     {
-        Fl::lock();
-        Fl::check();
-        Fl::unlock();
+        synth->getGuiMaster()->Log(synth->getRuntime().LogList.front());
+        synth->getRuntime().LogList.pop_front();
     }
+    // direct key actions (not for LV2)
+    /*if (!synth->getIsLV2Plugin() && Fl::event_ctrl() && Fl::event_alt() && keycheck == 0)
+    {
+        if (Fl::event_key('H')) // immediate shutdown
+            synth->getGuiMaster()->masterwindow->do_callback();
+        else if (Fl::event_key('R')) // resize to fit
+        {
+            if(!synth->getIsLV2Plugin())
+            for (unsigned int id = 0; id < 31; ++id)
+            {
+                SynthEngine *_synth = synth->getSynthFromId(id);
+                if (_synth != synth)
+                    _synth->getGuiMaster()->rescaleMain();
+            }
+            synth->getGuiMaster()->rescaleMain();
+        }
+        keycheck = 90; // we need this to stop a stream of repeats
+
+    }
+    else if (keycheck > 0)
+        --keycheck;*/
 }
 
 
@@ -220,14 +260,15 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
     unsigned char parameter = getData->data.parameter;
     unsigned char miscmsg = getData->data.miscmsg;
 
-//        cout << "Con " << int(control) << "  Kit " << int(kititem) << "  Eng " << int(engine) << "  Ins " << int(insert) << endl;
+//    synth->CBtest(getData);
 
-    fl_line_style(FL_SOLID); // probably not needed
-    if (control == TOPLEVEL::control::textMessage) // just show a message
+    if (control == TOPLEVEL::control::textMessage) // just show a non-modal message
     {
-        synth->getGuiMaster()->words->copy_label(textMsgBuffer.fetch(miscmsg).c_str());
-        synth->getGuiMaster()->cancel->hide();
-        synth->getGuiMaster()->message->show();
+        string name = textMsgBuffer.fetch(miscmsg);
+        if (name.empty())
+            synth->getGuiMaster()->message->hide();
+        else
+            synth->getGuiMaster()->setmessage(UNUSED, true, name, "Close");
         return;
     }
     if (npart == TOPLEVEL::section::scales)
@@ -257,7 +298,7 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
         return;
     }
 
-    if (kititem >= EFFECT::type::none && kititem != UNUSED) // effects
+    if (kititem >= (TOPLEVEL::insert::none | 128) && kititem != UNUSED) // effects
     {
         if (npart == TOPLEVEL::section::systemEffects)
         {
@@ -294,16 +335,26 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
         synth->getGuiMaster()->configui->returns_update(getData);
         return;
     }
+
     if (npart == TOPLEVEL::section::main && control == MAIN::control::exportPadSynthSamples) // special case
     {
         npart = parameter & 0x3f;
         getData->data.part = npart;
     }
+
     if (npart >= TOPLEVEL::section::main) // main / sys / ins
     {
         synth->getGuiMaster()->returns_update(getData);
         return;
     }
+    /*
+     * we are managing some part-related controls from here
+    */
+    if (npart < NUM_MIDI_PARTS && (kititem & engine & insert) == UNUSED)
+    {
+        if (synth->getGuiMaster()->part_group_returns(getData))
+            return;
+    }
 
     if (npart >= NUM_MIDI_PARTS)
         return; // invalid part number
@@ -311,12 +362,6 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
     if (kititem >= NUM_KIT_ITEMS && kititem != UNUSED)
         return; // invalid kit number
 
-    if (kititem == UNUSED && engine == UNUSED && insert == UNUSED && control == PART::control::defaultInstrument) // special case for part clear
-    {
-        synth->getGuiMaster()->returns_update(getData);
-        return;
-    }
-
     if (insert != UNUSED || (control != PART::control::enable && control != PART::control::instrumentName))
     {
         if (synth->getGuiMaster()->partui->partname == DEFAULT_NAME)
@@ -331,10 +376,11 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
         synth->getGuiMaster()->partui->returns_update(getData);
         return;
     }
-
+    if (kititem != synth->getGuiMaster()->partui->lastkititem)
+        return; // not for us!
     if (engine == PART::engine::padSynth) // padsynth
     {
-        if(synth->getGuiMaster()->partui->padnoteui)
+        if (synth->getGuiMaster()->partui->padnoteui)
         {
             switch (insert)
             {
@@ -363,7 +409,8 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
                         synth->getGuiMaster()->partui->padnoteui->filterui->returns_update(getData);
                     break;
                 case TOPLEVEL::insert::envelopeGroup:
-                case TOPLEVEL::insert::envelopePoints:
+                case TOPLEVEL::insert::envelopePointAdd:
+                case TOPLEVEL::insert::envelopePointDelete:
                 case TOPLEVEL::insert::envelopePointChange:
                     switch(parameter)
                     {
@@ -385,17 +432,17 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
                 case TOPLEVEL::insert::oscillatorGroup:
                 case TOPLEVEL::insert::harmonicAmplitude:
                 case TOPLEVEL::insert::harmonicPhaseBandwidth:
-                    if(synth->getGuiMaster()->partui->padnoteui->oscui)
+                    if (synth->getGuiMaster()->partui->padnoteui->oscui)
                         synth->getGuiMaster()->partui->padnoteui->oscui->returns_update(getData);
                     break;
                 case TOPLEVEL::insert::resonanceGroup:
                 case TOPLEVEL::insert::resonanceGraphInsert:
-                    if(synth->getGuiMaster()->partui->padnoteui->resui)
+                    if (synth->getGuiMaster()->partui->padnoteui->resui)
                         synth->getGuiMaster()->partui->padnoteui->resui->returns_update(getData);
                     break;
             }
         }
-        else if(miscmsg != NO_MSG)
+        else if (miscmsg != NO_MSG)
         {
             textMsgBuffer.fetch(miscmsg); // clear any text out.
         }
@@ -412,7 +459,8 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
                         synth->getGuiMaster()->partui->subnoteui->filterui->returns_update(getData);
                     break;
                 case TOPLEVEL::insert::envelopeGroup:
-                case TOPLEVEL::insert::envelopePoints:
+                case TOPLEVEL::insert::envelopePointAdd:
+                case TOPLEVEL::insert::envelopePointDelete:
                 case TOPLEVEL::insert::envelopePointChange:
                     switch(parameter)
                     {
@@ -478,7 +526,8 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
                     case TOPLEVEL::insert::envelopeGroup:
                         decode_envelope(synth, getData);
                         break;
-                    case TOPLEVEL::insert::envelopePoints:
+                    case TOPLEVEL::insert::envelopePointAdd:
+                    case TOPLEVEL::insert::envelopePointDelete:
                         decode_envelope(synth, getData);
                         break;
                     case TOPLEVEL::insert::envelopePointChange:
@@ -526,7 +575,8 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
                         synth->getGuiMaster()->partui->adnoteui->filterui->returns_update(getData);
                     break;
                 case TOPLEVEL::insert::envelopeGroup:
-                case TOPLEVEL::insert::envelopePoints:
+                case TOPLEVEL::insert::envelopePointAdd:
+                case TOPLEVEL::insert::envelopePointDelete:
                 case TOPLEVEL::insert::envelopePointChange:
                     switch(parameter)
                     {
@@ -555,6 +605,10 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
     }
 }
 
+// static string freqBPMStr(float val)
+// This has been moved as bpm2text in Misc/NumericFuncs.h with other BPM shared functions
+
+
 string convert_value(ValueType type, float val)
 {
     float f;
@@ -562,6 +616,12 @@ string convert_value(ValueType type, float val)
     string s;
     switch(type)
     {
+        case VC_plainReverse:
+            return(custom_value_units(127.0f - val,"",1));
+
+        case VC_pitchWheel:
+            return(custom_value_units(-val,"",1));
+
         case VC_percent127:
             return(custom_value_units(val / 127.0f * 100.0f+0.05f,"%",1));
 
@@ -574,6 +634,15 @@ string convert_value(ValueType type, float val)
         case VC_percent64_127:
             return(custom_value_units((val-64) / 63.0f * 100.0f+0.05f,"%",1));
 
+        case VC_PhaseOffset:
+            return(custom_value_units(val / 64.0f * 90.0f,"°",1));
+
+        case VC_WaveHarmonicMagnitude: {
+            const string unit = val > 0 ? "% (inverted)" : "%";
+            const int denom = val >= 0 ? 64 : -63;
+            return(custom_value_units(val / denom * 100.0f,unit,1));
+        }
+
         case VC_GlobalFineDetune:
             return(custom_value_units((val-64),"cents",1));
 
@@ -581,11 +650,14 @@ string convert_value(ValueType type, float val)
             return(custom_value_units((val-96.0f)/96.0f*40.0f,"dB",1));
 
         case VC_LFOfreq:
-            f = (powf(2.0f, val * 10.0f) - 1.0f) / 12.0f;
+            f = (power<2>(val * 10.0f) - 1.0f) / 12.0f;
             return variable_prec_units(f, "Hz", 3);
 
+        case VC_LFOfreqBPM:
+            return bpm2text(val);
+
         case VC_LFOdepthFreq: // frequency LFO
-            f=powf(2.0f,(int)val/127.0f*11.0f)-1.0f;
+            f=power<2>((int)val/127.0f*11.0f)-1.0f;
             return variable_prec_units(f, "cents", 2);
 
         case VC_LFOdepthAmp: // amplitude LFO
@@ -601,24 +673,25 @@ string convert_value(ValueType type, float val)
             f = ((int)val) / 127.0f * 4.0f + 0.005f;
             return(custom_value_units(f,"s",2));
 
-        case VC_LFOstartphase:
-            if((int)val == 0)
+        case VC_LFOstartphaseRand:
+            if ((int)val == 0)
                 return("random");
-            else
-                return(custom_value_units(((int)val - 64.0f) / 127.0f
-                                      * 360.0f, "°"));
+            // fallthrough
+        case VC_LFOstartphase:
+            return(custom_value_units(((int)val - 64.0f) / 127.0f
+                                  * 360.0f, "°"));
         case VC_EnvelopeDT:
             // unfortunately converttofree() is not called in time for us to
             // be able to use env->getdt(), so we have to compute ourselves
-            f = (powf(2.0f, ((int)val) / 127.0f * 12.0f) - 1.0f) * 10.0f;
-            if(f >= 1000)
+            f = (power<2>(((int)val) / 127.0f * 12.0f) - 1.0f) * 10.0f;
+            if (f >= 1000)
                 return variable_prec_units(f/1000.0f, "s", 2);
             else
                 return variable_prec_units(f, "ms", 2);
 
         case VC_EnvelopeFreqVal:
-            f=(powf(2.0f, 6.0f * fabsf((int)val - 64.0f) / 64.0f) -1.0f) * 100.0f;
-            if((int)val<64) f = -f;
+            f=(power<2>(6.0f * fabsf((int)val - 64.0f) / 64.0f) -1.0f) * 100.0f;
+            if ((int)val<64) f = -f;
             return variable_prec_units(f, "cents", 2);
 
         case VC_EnvelopeFilterVal:
@@ -636,18 +709,18 @@ string convert_value(ValueType type, float val)
             return variable_prec_units(f, "dB", 2);
 
         case VC_EnvelopeBandwidthVal:
-	    f = powf(2.0f, 10.0f * ((int)val - 64) / 64.0f);
+	    f = power<2>(10.0f * ((int)val - 64) / 64.0f);
             return variable_prec_units(f, "x", 4);
 
         case VC_FilterFreq0: // AnalogFilter
-            f=powf(2.0f, (val / 64.0f - 1.0f) * 5.0f + 9.96578428f);
+            f=power<2>((val / 64.0f - 1.0f) * 5.0f + 9.96578428f);
             if (f >= 1000.0f)
                 return variable_prec_units(f/1000.0f, "kHz", 2);
             else
                 return variable_prec_units(f, "Hz", 2);
 
         case VC_FilterFreq2: // SVFilter
-            f=powf(2.0f, (val / 64.0f - 1.0f) * 5.0f + 9.96578428f);
+            f=power<2>((val / 64.0f - 1.0f) * 5.0f + 9.96578428f);
             // We have to adjust the freq because of this line
             // in method SVFilter::computefiltercoefs() (file SVFilter.cpp)
             //
@@ -655,7 +728,7 @@ string convert_value(ValueType type, float val)
             //
             // Using factor 4.0 instead of the usual 2.0*PI leads to a
             // different effective cut-off freq, which we will be showing
-            f *= 4.0 / (2.0 * PI);
+            f *= 4.0 / TWOPI;
             if (f >= 1000.0f)
                 return variable_prec_units(f/1000.0f, "kHz", 2);
             else
@@ -676,8 +749,8 @@ string convert_value(ValueType type, float val)
 
         case VC_FilterVelocityAmp:
             val = (int)val / 127.0 * -6.0; // formant offset value
-            f = powf(2.0f,val + log(1000.0f)/log(2.0f)); // getrealfreq
-            f = log(f/1000.0f)/log(powf(2.0f,1.0f/12.0f))*100.0f; // in cents
+            f = power<2>(val + log(1000.0f)/log(2.0f)); // getrealfreq
+            f = log(f/1000.0f)/log(power<2>(1.0f/12.0f))*100.0f; // in cents
             return custom_value_units(f-0.5, "cents") +
                    "\n(Formant offset: " + custom_value_units(val, "x stretch)",2);
 
@@ -696,7 +769,7 @@ string convert_value(ValueType type, float val)
             return(s);
 
         case VC_FormFilterClearness:
-            f = powf(10.0f, (val - 32.0f) / 48.0f);
+            f = power<10>((val - 32.0f) / 48.0f);
             return custom_value_units(f, " switch rate",2);
 
         case VC_FormFilterSlowness:
@@ -717,10 +790,10 @@ string convert_value(ValueType type, float val)
                 return(custom_value_units(-60.0f*(1.0f-lrint(val)/127.0f),"dB",1));
 
         case VC_ADDVoiceDelay:
-            if((int) val == 0)
+            if ((int) val == 0)
                 return "No delay";
             f = (expf((val/127.0f) * logf(50.0f)) - 1) / 10;
-            if(f >= 1)
+            if (f >= 1)
                 return variable_prec_units(f, "s", 2, true);
             else
                 return variable_prec_units(f * 1000, "ms", 1);
@@ -740,6 +813,14 @@ string convert_value(ValueType type, float val)
             else
                 return(custom_value_units((val-96.0f)/96.0f*40.0f,"dB",1));
 
+        case VC_PartHumaniseDetune:
+            s = "Detune: ";
+            i = (int) val;
+            if (i == 0)
+                return s + "disabled";
+            else
+                return s + "between 0 and " + to_string(i) + " cents";
+
         case VC_PartHumaniseVelocity:
             s = "Attenuation: ";
             i = (int) val;
@@ -749,28 +830,20 @@ string convert_value(ValueType type, float val)
                 return s + "between 0 and " + to_string(i) + "%";
 
         case VC_PanningRandom:
-            i = lrint(val);
-            if(i==0)
-                return("random");
-            else if(i==64)
-                return("centered");
-            else if(i<64)
-                return(custom_value_units((64.0f - i) / 63.0f * 100.0f,"% left"));
-            else
-                return(custom_value_units((i - 64.0f)/63.0f*100.0f,"% right"));
+            return(custom_value_units(val / 63.0f * 100.0f,"%"));
 
         case VC_PanningStd:
             i = lrint(val);
-            if(i==64)
+            if (i==64)
                 return("centered");
-            else if(i<64)
+            else if (i<64)
                 return(custom_value_units((64.0f - i) / 64.0f * 100.0f,"% left"));
             else
                 return(custom_value_units((i - 64.0f)/63.0f*100.0f,"% right"));
 
         case VC_EnvStretch:
             s.clear();
-            f = powf(2.0f,(int)val/64.0f);
+            f = power<2>((int)val/64.0f);
             s += custom_value_units((int)val/127.0f*100.0f+0.05f,"%",1);
             if ((int)val!=0)
             {
@@ -783,7 +856,7 @@ string convert_value(ValueType type, float val)
             s.clear();
             i = val;
             i = (i == 0) ? 1 : (i); // val == 0 is not allowed
-            f = powf(2.0f,(i-64.0)/63.0f);
+            f = power<2>((i-64.0)/63.0f);
             s += custom_value_units((i-64.0f)/63.0f*100.0f,"%");
             if (i != 64)
             {
@@ -799,13 +872,13 @@ string convert_value(ValueType type, float val)
             return(custom_value_units(f+((f<0) ? (-0.005f) : (0.005f)),"Hz",2));
 
         case VC_FixedFreqET:
-            f = powf(2.0f, (lrint(val) - 1) / 63.0f) - 1.0f;
-            if(lrint(val) <= 1) /* 0 and 1 are both fixed */
+            f = power<2>((lrint(val) - 1) / 63.0f) - 1.0f;
+            if (lrint(val) <= 1) /* 0 and 1 are both fixed */
                 return "Fixed";
-            else if(lrint(val) <= 64)
-                return custom_value_units(powf(2.0f,f),"x /octave up",2);
+            else if (lrint(val) <= 64)
+                return custom_value_units(power<2>(f),"x /octave up",2);
             else
-                return custom_value_units(powf(3.0f,f),"x /octave up",2);
+                return custom_value_units(power<3>(f),"x /octave up",2);
 
         case VC_FilterGain:
             f = ((int)val / 64.0f -1.0f) * 30.0f; // -30..30dB
@@ -820,18 +893,18 @@ string convert_value(ValueType type, float val)
                 s += "Velocity sensing disabled.";
                 return(s);
             }
-            f = powf(8.0f, (64.0f - (float)i) / 64.0f);
+            f = power<8>((64.0f - (float)i) / 64.0f);
             // Max dB range for vel=1 compared to vel=127
             s += "Velocity Dynamic Range ";
             f = -20.0f * logf(powf((1.0f / 127.0f), f)) / log(10.0f);
             s += variable_prec_units(f, "dB", 2);
             s += "\nVelocity/2 = ";
-            s += variable_prec_units(f/(-1 * log2(127)), "dB", 2);
+            s += variable_prec_units(f/(-1 * std::log2(127)), "dB", 2);
             return(s);
 
         case VC_BandWidth:
             f = powf((int)val / 1000.0f, 1.1f);
-            f = powf(10.0f, f * 4.0f) * 0.25f;
+            f = power<10>(f * 4.0f) * 0.25f;
             return variable_prec_units(f, "cents", 2);
 
         case VC_SubBandwidth:
@@ -839,30 +912,65 @@ string convert_value(ValueType type, float val)
                Considering the variability of the synthesis depending
                on number of filter stages, it seems accurate enough.
              */
-	    f = powf(10.0f, (val - 127.0f) / 127.0f * 4.0f) * 4800;
+	    f = power<10>((val - 127.0f) / 127.0f * 4.0f) * 4800;
             return variable_prec_units(f, "cents", 3);
 
         case VC_SubBandwidthRel:
-	    f = powf(100.0f, (63 - (int)val) / 64.0f);
+	    f = power<100>(val / 64.0f);
             return variable_prec_units(f, "x", 3);
 
+        case VC_SubHarmonicMagnitude:
+            return custom_value_units(val / 127.0f * 100.0f, "%", 1);
+
         case VC_SubBandwidthScale:
-            if((int)val == 0)
+            if ((int)val == 0)
                 return "Constant";
-	    f = val / 64.0f * 3.0f;
+            f = val / 64.0f * 3.0f;
             return "Factor (100,10k): " +
-                variable_prec_units(powf(10,f), "", 4) + ", " +
+                variable_prec_units(power<10>(f), "", 4) + ", " +
                 variable_prec_units(powf(0.1,f), "x", 4);
 
+        case VC_XFadeUpdate:
+        {
+            unsigned int millisec = logDial2millisec(val);
+            if (millisec > 1000)
+                return variable_prec_units(float(millisec) / 1000, "sec", 1);
+            if (millisec > 0)
+                return variable_prec_units(float(millisec), "ms", 0);
+            else
+                return "off";
+            break;
+        }
+
+        case VC_Retrigger:
+        {
+            if (val > 0) val += 2300;
+            // in the UI we remove a socket of 200ms from the dial setting,
+            // to prevent the user from choosing overly fast retriggering
+            // 200ms correspond to the log10 setting of 2300
+            return convert_value(VC_XFadeUpdate, val);
+            break;
+        }
+
+        case VC_RandWalkSpread:
+        {
+            double spread = RandomWalk::param2spread(val);
+            if (spread > 1)
+                return variable_prec_units((spread - 1) * 100.0, "%", 1);
+            else
+                return "no random walk.";
+            break;
+        }
+
         case VC_FilterVelocitySense: // this is also shown graphically
-            if((int)val==127)
+            if ((int)val==127)
                 return("off");
             else
                 return(custom_value_units(val,""));
             break;
 
         case VC_FXSysSend:
-            if((int)val==0)
+            if ((int)val==0)
                 return("-inf dB");
             else
                 return(custom_value_units((val-96.0f)/96.0f*40.0f,"dB",1));
@@ -888,7 +996,7 @@ string convert_value(ValueType type, float val)
             s.clear();
             // ToDo: It would be nice to calculate the ratio between left
             // and right. We would need to know the delay time however...
-            f = (powf(2.0f, fabsf((int)val-64.0f)/64.0f*9.0f)-1.0); // ms
+            f = (power<2>(fabsf((int)val-64.0f)/64.0f*9.0f)-1.0); // ms
             if ((int)val < 64)
             {
                 s+="left +"+custom_value_units(f+0.05,"ms",1)+" / ";
@@ -904,7 +1012,7 @@ string convert_value(ValueType type, float val)
         case VC_FXEchoDW:
             s.clear();
             f = (int)val / 127.0f;
-            if(f < 0.5f)
+            if (f < 0.5f)
             {
                 f = f * 2.0f;
                 f *= f;  // for Reverb and Echo
@@ -928,7 +1036,7 @@ string convert_value(ValueType type, float val)
             return(custom_value_units(f,"dB",1));
 
         case VC_FXReverbTime:
-            f = powf(60.0f, (int)val / 127.0f) - 0.97f; // s
+            f = power<60>((int)val / 127.0f) - 0.97f; // s
             return variable_prec_units(f, "s", 2, true);
 
         case VC_FXReverbIDelay:
@@ -964,7 +1072,7 @@ string convert_value(ValueType type, float val)
         case VC_FXReverbDW:
             s.clear();
             f = (int)val / 127.0f;
-            if(f < 0.5f)
+            if (f < 0.5f)
             {
                 f = f * 2.0f;
                 f *= f;  // for Reverb and Echo
@@ -991,15 +1099,18 @@ string convert_value(ValueType type, float val)
             return(custom_value_units(f,"dB",1));
 
         case VC_FXlfofreq:
-            f = (powf(2.0f, (int)val / 127.0f * 10.0f) - 1.0f) * 0.03f;
+            f = (power<2>((int)val / 127.0f * 10.0f) - 1.0f) * 0.03f;
             return variable_prec_units(f, "Hz", 3);
 
+        case VC_FXlfofreqBPM:
+            return bpm2text(val / 127.0f);
+
         case VC_FXChorusDepth:
-            f = powf(8.0f, ((int)val / 127.0f) * 2.0f) -1.0f; //ms
+            f = power<8>(((int)val / 127.0f) * 2.0f) -1.0f; //ms
             return variable_prec_units(f, "ms", 2, true);
 
         case VC_FXChorusDelay:
-            f = powf(10.0f, ((int)val / 127.0f) * 2.0f) -1.0f; //ms
+            f = power<10>(((int)val / 127.0f) * 2.0f) -1.0f; //ms
             return variable_prec_units(f, "ms", 2, true);
 
         case VC_FXdefaultFb:
@@ -1018,7 +1129,7 @@ string convert_value(ValueType type, float val)
         case VC_FXdefaultDW:
             s.clear();
             f = (int)val / 127.0f;
-            if(f < 0.5f)
+            if (f < 0.5f)
             {
                 f = f * 2.0f;
                 f = 20.0f * logf(f) / logf(10.0f);
@@ -1035,14 +1146,14 @@ string convert_value(ValueType type, float val)
             return(s);
 
         case VC_FXEQfreq:
-            f = 600.0f * powf(30.0f, ((int)val - 64.0f) / 64.0f);
+            f = 600.0f * power<30>(((int)val - 64.0f) / 64.0f);
             if (f >= 1000)
                 return variable_prec_units(f/1000.f, "kHz", 2);
             else
                 return variable_prec_units(f, "Hz", 2, true);
 
         case VC_FXEQq:
-            f = powf(30.0f, ((int)val - 64.0f) / 64.0f);
+            f = power<30>(((int)val - 64.0f) / 64.0f);
             return variable_prec_units(f, "", 3, true);
 
         case VC_FXEQgain:
@@ -1111,6 +1222,7 @@ void custom_graph_dimensions(ValueType vt, int& w, int& h)
     default:
         w = 0;
         h = 0;
+        break;
     }
 }
 
@@ -1120,13 +1232,13 @@ inline void grid(int x, int y, int w, int h, int sections)
 
         int j = 1;
         int gDist = h / sections;
-        for(; j < sections; j++) /* Vertical */
+        for (; j < sections; j++) /* Vertical */
         {
             fl_line(x, y - gDist * j, x + w, y - gDist * j);
         }
 
         gDist = w / sections;
-        for(j = 1; j < sections; j++) /* Horizontal */
+        for (j = 1; j < sections; j++) /* Horizontal */
         {
             fl_line(x + gDist * j, y, x + gDist * j, y - h);
         }
@@ -1145,7 +1257,7 @@ void custom_graphics(ValueType vt, float val,int W,int H)
     {
     case VC_FilterVelocitySense:
     {
-        p = powf(8.0f,(64.0f-(int)val)/64.0f);
+        p = power<8>((64.0f-(int)val)/64.0f);
 
         /* Grid */
         grid(x0,y0,_w,_h, 4);
@@ -1159,7 +1271,7 @@ void custom_graphics(ValueType vt, float val,int W,int H)
         else
         {
             fl_begin_line();
-            for(i = 0; i < _w; i++)
+            for (i = 0; i < _w; i++)
             {
                 x = (float)i / (float)_w;
                 y = powf(x,p) * _h;
@@ -1171,13 +1283,13 @@ void custom_graphics(ValueType vt, float val,int W,int H)
     }
     case VC_FormFilterClearness:
     {
-        p = powf(10.0f, (val - 32.0f) / 48.0f); //clearness param
+        p = power<10>((val - 32.0f) / 48.0f); //clearness param
         grid(x0,y0,_w,_h,10);
         fl_color(FL_BLUE);
         fl_begin_line();
         x = 0;
         float frac = 1.0f / (float)_w;
-        for(i = 0; i < _w; i++)
+        for (i = 0; i < _w; i++)
         {
             y = (atanf((x * 2.0f - 1.0f) * p) / atanf(p) + 1.0f) * 0.5f * _h;
             fl_vertex((float)x0 + i, (float)y0 - y);
@@ -1210,7 +1322,7 @@ void custom_graphics(ValueType vt, float val,int W,int H)
         /* Scale lines */
 
         fl_font(fl_font(),8);
-        for(i = 0; i < 4; i++) /* 10x / 10%, 100x / 1% ... */
+        for (i = 0; i < 4; i++) /* 10x / 10%, 100x / 1% ... */
         {
             y = ry * (i + 1);
             fl_color(169,169,169);
@@ -1227,13 +1339,13 @@ void custom_graphics(ValueType vt, float val,int W,int H)
 
         fl_color(196,196,196); /* Lighter inner lines*/
 
-        for(i = 10;i != 0; i *= 10)
+        for (i = 10;i != 0; i *= 10)
         {
-            for(j = 2; j < 10; j++)
+            for (j = 2; j < 10; j++)
             {
                 x = x0 + rx * (log10(i * j) - lg1020) + 1;
                 fl_line(x, y0, x, y0 - _h);
-                if(i * j >= 20000)
+                if (i * j >= 20000)
                 {
                     i = 0;
                     break;
@@ -1242,7 +1354,7 @@ void custom_graphics(ValueType vt, float val,int W,int H)
         }
 
         fl_font(fl_font(),10);
-        for(i = 0; i < 4; i++) /* 20, 100, 1k, 10k */
+        for (i = 0; i < 4; i++) /* 20, 100, 1k, 10k */
         {
             x = x0 + (i == 0 ?  0 : ((float)i + 1 - lg1020) * rx);
             fl_color(127,127,127); /* Darker boundary lines */
@@ -1260,7 +1372,7 @@ void custom_graphics(ValueType vt, float val,int W,int H)
 
         /* Function curve */
         fl_color(FL_BLUE);
-        if((int)val == 0)
+        if ((int)val == 0)
         {
             fl_line(x0, cy, x0 + _w, cy);
         }
@@ -1277,7 +1389,7 @@ void custom_graphics(ValueType vt, float val,int W,int H)
 
             cairo_set_source_rgb(cr, 1, 0, 0);
             cairo_set_line_width(cr, 1.5);
-            cairo_move_to(cr, x0, cy - ry * log10(powf(50, p)));
+            cairo_move_to(cr, x0, cy - ry * log10(power<50>(p)));
             cairo_line_to(cr, x0 + _w, cy - ry * log10(powf(0.05, p)));
             cairo_stroke(cr);
 
@@ -1290,31 +1402,31 @@ void custom_graphics(ValueType vt, float val,int W,int H)
     }
 }
 
-string variable_prec_units(float v, string u, int maxPrec, bool roundup)
+string variable_prec_units(float v, const string& u, int maxPrec, bool roundup)
 {
     int digits = 0, lim = (int) pow(10,maxPrec);
     float _v = fabsf(v);
-    while(maxPrec > digits)
+    while (maxPrec > digits)
     {
-        if(_v >= lim)
+        if (_v >= lim)
             break;
         digits++;
         lim /= 10;
     }
     if (roundup)
     {
-        v += 5 * powf(10,-(digits + 1));
+        v += 5 * power<10>(-(digits + 1));
     }
     return custom_value_units(v, u, digits);
 }
 
-string custom_value_units(float v, string u, int prec)
+string custom_value_units(float v, const string& u, int prec)
 {
     ostringstream oss;
     oss.setf(std::ios_base::fixed);
     oss.precision(prec);
     oss << v << " " << u;
-    return(string(oss.str()));
+    return(oss.str());
 }
 
 ValueType getLFOdepthType(int group)
@@ -1328,6 +1440,11 @@ ValueType getLFOdepthType(int group)
     return(VC_plainValue);
 }
 
+ValueType getLFOFreqType(int bpmEnabled)
+{
+    return (bpmEnabled == 0) ? VC_LFOfreq : VC_LFOfreqBPM;
+}
+
 ValueType getFilterFreqType(int type)
 {
     switch(type)
@@ -1347,3 +1464,17 @@ ValueType getFilterFreqTrackType(int offset)
         default: return(VC_FilterFreqTrack1);
     }
 }
+
+/** convert a millesconds value to a logarithmic dial setting */
+int millisec2logDial(unsigned int ms)
+{
+    return ms==0? -1
+                : log10f(float(ms)) * 1000;
+}
+
+/** convert setting from a logarithmic dial back to millesconds */
+unsigned int logDial2millisec(int dial)
+{
+    return dial<0? 0
+                 : power<10>(dial / 1000.0f) + 0.5;
+}
