@@ -1,10 +1,10 @@
 /*
     globals.h - general static definitions
 
-    Copyright 2018-2019, Will Godfrey
+    Copyright 2018-2022, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -24,6 +24,10 @@
 #include <cstdint>
 #include <cstddef>
 #include <string>
+#include <iostream>
+
+typedef unsigned char uchar;
+typedef unsigned int  uint;
 
 /*
  * For test purposes where you want guaranteed identical results, enable the
@@ -46,135 +50,117 @@
 #define Fmul2I 1073741823
 #define Cshift2I 23
 
-/*
- * proposed conversions from float to hi res int
- * multiplier is 1000000
- *
- * for LFO freq turns actual value 85.25 into 85250000
- * current step size 0.06 becomes 6000
- *
- * scales reference frequency 440Hz becomes 440000000
- * At 20Hz resolution is still better than 1/1000 cent
- * Assumed detectable interval is 5 cents
- *
- * also use for integers that need higher resolution
- * such as unspecified 0-127 integers
-*/
-
 // many of the following are for convenience and consistency
 // changing them is likely to have unpredicable consequences
 
 // sizes
-const unsigned char COMMAND_SIZE = 252;
-const unsigned char MAX_HISTORY = 25;
-const int MAX_PRESETS = 1000;
-const unsigned char MAX_PRESET_DIRS = 128;
-const unsigned char MAX_BANK_ROOT_DIRS = 128;
-const unsigned char MAX_BANKS_IN_ROOT = 128;
-const unsigned char MAX_INSTRUMENTS_IN_BANK = 160;
-const unsigned char MAX_AD_HARMONICS = 128;
-const unsigned char MAX_SUB_HARMONICS = 64;
-const unsigned char PAD_MAX_SAMPLES = 96;
-const unsigned char NUM_MIDI_PARTS = 64;
-const unsigned char PART_POLY = 0;
-const unsigned char PART_MONO = 1;
-const unsigned char PART_LEGATO = 2;
-const unsigned char MIDI_NOT_LEGATO = 3;
-const unsigned char MIDI_LEGATO = 4;
-const unsigned char NUM_MIDI_CHANNELS = 16;
-const unsigned char MIDI_LEARN_BLOCK = 200;
-const int MAX_ENVELOPE_POINTS = 40;
-const int MIN_ENVELOPE_DB = -60;
-const int MAX_RESONANCE_POINTS = 256;
-const int MAX_KEY_SHIFT = 36;
-const int MIN_KEY_SHIFT = -36;
-const float A_MIN = 30.0f;
-const float A_DEF = 440.0f;
-const float A_MAX = 1100.0f;
+#define COMMAND_SIZE 252
+#define MAX_HISTORY 25
+#define MAX_PRESETS 1000
+#define MAX_PRESET_DIRS 128
+#define MAX_BANK_ROOT_DIRS 128
+#define MAX_BANKS_IN_ROOT 128
+#define MAX_INSTRUMENTS_IN_BANK 160
+#define MAX_AD_HARMONICS 128
+#define MAX_SUB_HARMONICS 64
+#define NUM_MIDI_PARTS 64
+#define PART_NORMAL 0
+#define PART_MONO 1
+#define PART_LEGATO 2
+#define MIDI_NOT_LEGATO 3
+#define MIDI_LEGATO 4
+#define NUM_MIDI_CHANNELS 16
+#define MIDI_LEARN_BLOCK 400
+#define MAX_ENVELOPE_POINTS 40
+#define MIN_ENVELOPE_DB -60
+#define MAX_RESONANCE_POINTS 256
+#define MAX_KEY_SHIFT 36
+#define MIN_KEY_SHIFT -36
+#define A_MIN 30.0f
+#define A_DEF 440.0f
+#define A_MAX 1100.0f
 
+// There is nothing which technically prevents these from being lower or higher,
+// but we need to set the limits for the UI somewhere.
+#define BPM_FALLBACK_MIN 32.0f
+#define BPM_FALLBACK_MAX 480.0f
 
-const unsigned int MIN_OSCIL_SIZE = 256; // MAX_AD_HARMONICS * 2
-const unsigned int MAX_OSCIL_SIZE = 16384;
-const unsigned int MIN_BUFFER_SIZE = 16;
-const unsigned int MAX_BUFFER_SIZE = 8192;
-const unsigned char NO_MSG = 255; // these two may become different
-const unsigned char UNUSED = 255;
+// The number of discrete steps we use for the LFO BPM frequency. Make sure to
+// update LFO_BPM_LCM as well, if this is updated.
+#define LFO_BPM_STEPS 33
+// The Least Common Multiple of all the possible LFO fractions.
+#define LFO_BPM_LCM 720720
+
+#define MIN_OSCIL_SIZE 256 // MAX_AD_HARMONICS * 2
+#define MAX_OSCIL_SIZE 16384
+#define MIN_BUFFER_SIZE 16
+#define MAX_BUFFER_SIZE 8192
+#define NO_MSG 255 // these two may become different
+#define UNUSED 255
 
 // GUI colours
-const unsigned int ADD_COLOUR = 0xdfafbf00;
-const unsigned int BASE_COLOUR = 0xbfbfbf00;
-const unsigned int SUB_COLOUR = 0xafcfdf00;
-const unsigned int PAD_COLOUR = 0xcfdfaf00;
-const unsigned int YOSHI_COLOUR = 0x0000e100;
-const unsigned int EXTOSC_COLOUR = 0x8fbfdf00;
-const unsigned int EXTVOICE_COLOUR = 0x9fdf8f00;
-const unsigned int MODOFF_COLOUR = 0x80808000;
+#define ADD_COLOUR 0xdfafbf00
+#define BASE_COLOUR 0xfdf6e600
+#define SUB_COLOUR 0xafcfdf00
+#define PAD_COLOUR 0xcdddad00
+#define YOSHI_COLOUR 0x0000e100
+#define EXTOSC_COLOUR 0x8fbfdf00
+#define EXTVOICE_COLOUR 0x9fdf8f00
+#define MODOFF_COLOUR 0x80808000
 
 // these were previously (pointlessly) user configurable
-const unsigned char NUM_VOICES = 8;
-const unsigned char POLIPHONY = 80;
-const unsigned char PART_POLIPHONY = 60;
-const unsigned char NUM_SYS_EFX = 4;
-const unsigned char NUM_INS_EFX = 8;
-const unsigned char NUM_PART_EFX = 3;
-const unsigned char NUM_KIT_ITEMS = 16;
-const float VELOCITY_MAX_SCALE = 8.0f;
-const unsigned char FADEIN_ADJUSTMENT_SCALE = 20;
-const unsigned char MAX_EQ_BANDS = 8;  // MAX_EQ_BANDS must be less than 20
-const unsigned char MAX_FILTER_STAGES = 5;
-const unsigned char FF_MAX_VOWELS = 6;
-const unsigned char FF_MAX_FORMANTS = 12;
-const unsigned char FF_MAX_SEQUENCE = 8;
-const unsigned char MAX_PHASER_STAGES = 12;
-const unsigned char MAX_ALIENWAH_DELAY = 100;
+#define NUM_VOICES 8
+#define POLYPHONY 60
+#define PART_DEFAULT_LIMIT 20
+#define NUM_SYS_EFX 4
+#define NUM_INS_EFX 8
+#define NUM_PART_EFX 3
+#define NUM_KIT_ITEMS 16
+#define FADEIN_ADJUSTMENT_SCALE 20
+#define MAX_EQ_BANDS 8  // MAX_EQ_BANDS must be less than 20
+#define MAX_FILTER_STAGES 5
+#define FF_MAX_VOWELS 6
+#define FF_MAX_FORMANTS 12
+#define FF_MAX_SEQUENCE 8
 
-const std::string DEFAULT_NAME = "Simple Sound";
-const std::string UNTITLED = "No Title";
+#define DEFAULT_NAME "Simple Sound"
+#define UNTITLED "No Title"
 
-const unsigned char FORCED_EXIT = 16;
+#define DEFAULT_AUDIO alsa_audio
+#define DEFAULT_MIDI alsa_midi
 
+#define FORCED_EXIT 16
 
-enum muteState {Idle, Pending, Fading, Active, Complete, Request, Immediate};
-
-enum Session : unsigned char {
-    Normal = 0,
-    Default,
-    JackFirst,
-    JackSecond,
-    StartupFirst,
-    StartupSecond,
-    InProgram,
-    RestoreConf
-};
-
-namespace YOSH
+namespace _SYS_
 {
     // float to bool done this way to ensure consistency
     // we are always using positive values
     inline bool F2B(float value) {return value > 0.5f;}
+
+    enum mute {Idle, Pending, Fading, Active, Complete, Request, Immediate};
+
+    // session types and stages
+    enum type {Normal, Default, JackFirst, JackSecond, StartupFirst, StartupSecond, InProgram, RestoreConf};
+
+    // Log types
+    const char LogNormal = 0;
+    const char LogError = 1;
+    const char LogNotSerious = 2;
 }
 
 /*
- * for many of the following, where they are in groups the
+ * For many of the following, where they are in groups the
  * group order must not change, but the actual values can
  * and new entries can be added between the group ends
  */
 
-
-namespace ENVMODE
-{
-    const unsigned char amplitudeLin = 1;
-    const unsigned char amplitudeLog = 2;
-    const unsigned char frequency = 3;
-    const unsigned char filter = 4;
-    const unsigned char bandwidth = 5;
-}
-
 namespace TOPLEVEL // usage TOPLEVEL::section::vector
 {
     enum section : unsigned char {
-        part1 = 0,
-        part64 = 63,
+        part1 = 0,   // nothing must come
+        part64 = 63, // between these two
+
+        undoMark = 68, // 44
         copyPaste = 72, // 48 (not yet!)
         vector = 192, // CO
         midiLearn = 216, // D8
@@ -189,58 +175,87 @@ namespace TOPLEVEL // usage TOPLEVEL::section::vector
     };
 
     namespace type {
-        // bits 0, 1
-        const unsigned char Adjust = 0; // return value adjusted within limits
-        const unsigned char Read = 0; // i.e. !write
-        const unsigned char Minimum = 1; // return this value
-        const unsigned char Maximum = 2; // return this value
-        const unsigned char Default = 3; // return this value
-        // remaining used bit-wise
-        const unsigned char Limits = 4; // read above limits
-        const unsigned char Error = 8;
-        const unsigned char LearnRequest = 16;
-        const unsigned char Learnable = 32;
-        const unsigned char Write = 64;
-        const unsigned char Integer = 128; // false = float
+        enum {
+            // bits 0, 1 as values
+            Adjust = 0, // return value adjusted within limits
+            Minimum, // return this value
+            Maximum, // return this value
+            Default, // return this value
+            // remaining used bit-wise
+            Limits, // read limits shown above
+            Error = 8,
+            LearnRequest = 16,
+            Learnable = 32,
+            Write = 64,
+            Integer = 128 // false = float
+        };
     }
 
     namespace action {
-        // bits 0 to 3
-        const unsigned char toAll = 0; // except MIDI
-        const unsigned char fromMIDI = 1;
-        const unsigned char fromCLI = 2;
-        const unsigned char fromGUI = 3;
-        // space for any other sources
-        const unsigned char noAction = 15; // internal use
-        // remaining used bit-wise
-        const unsigned char forceUpdate = 32;
-        const unsigned char loop = 64; // internal use
-        const unsigned char lowPrio = 128;
-        const unsigned char muteAndLoop = 192;
+        enum {
+            // bits 0 to 3
+            toAll = 0, // except MIDI
+            fromMIDI,
+            fromCLI,
+            fromGUI,
+            // space for any other sources
+            noAction = 15, // internal use
+            // remaining used bit-wise
+            forceUpdate = 32,
+            loop = 64, // internal use
+            lowPrio = 128,
+            muteAndLoop = 192
+        };
     }
 
     enum control : unsigned char {
         // insert any new entries here
-        textMessage = 254, // FE
+
+        /*
+         * the following values must never appear in any other sections
+         */
+        unrecognised = 253, // FD
+        textMessage,
         forceExit // this is effective from *any* section!
     };
 
+    enum msgResponse : unsigned char {
+        refreshBankDefaults,
+        cancelBankDefaults,
+        cancelMidiLearn
+        // any other value = no response
+        // but there may still be a message
+    };
+
     // inserts are here as they are split between many
     // sections but must remain distinct.
     enum insert : unsigned char {
-        LFOgroup = 0,
+        none = 0,
+        reverb,
+        echo,
+        chorus,
+        phaser,
+        alienWah,
+        distortion,
+        eq,
+        dynFilter,
+        // any new effects should go here
+        count, // this must be the last type!
+
+        LFOgroup, // 10
         filterGroup,
         envelopeGroup,
-        envelopePoints, // this should be split in two
+        envelopePointAdd,
+        envelopePointDelete,
         envelopePointChange,
-        oscillatorGroup, // 5
+        oscillatorGroup,
         harmonicAmplitude,
-        harmonicPhaseBandwidth, // this should also be split in two
+        harmonicPhaseBandwidth, // this should be split in two
         resonanceGroup,
         resonanceGraphInsert,
-        systemEffectSend = 16,
+        systemEffectSend,
         partEffectSelect,
-        kitGroup = 32
+        kitGroup //23
     };
 
     enum insertType : unsigned char {
@@ -256,13 +271,20 @@ namespace TOPLEVEL // usage TOPLEVEL::section::vector
         Scale, //      complete Microtonal settings
         State, //      entire system State
         Vector, //     per channel Vector settings
-        // insert any new lists here
         MLearn, //     learned MIDI CC lists
-        Config, // only file types from here onwards
+        Presets, //    parts of instruments or effects
+
+        // not XML but there for consistency
+        PadSample,
+        ScalaTune,
+        ScalaMap,
+        Dir, // for filer, any directory request
+
+        // only file types from here onwards
+        Config,
         MasterConfig,
-        Presets,
         Bank,
-        History,
+        History
     };
 }
 
@@ -275,8 +297,8 @@ namespace CONFIG // usage CONFIG::control::oscillatorSize
         virtualKeyboardLayout,
         XMLcompressionLevel,
         reportsDestination,
+        logTextSize,
         savedInstrumentFormat,
-        showEnginesTypes,
         defaultStateStart = 16,
         hideNonFatalErrors,
         showSplash,
@@ -289,6 +311,8 @@ namespace CONFIG // usage CONFIG::control::oscillatorSize
         enableSinglePath,
         historyLock,
         exposeStatus, // CLI only
+        enableHighlight, // in banks
+        handlePadSynthBuild,   // how to build PADSynth wavetable; 0=legacy/muted, 1=background thread, 2=autoApply
 
         // start of engine controls
         jackMidiSource = 32,
@@ -302,6 +326,9 @@ namespace CONFIG // usage CONFIG::control::oscillatorSize
         alsaAudioDevice,
         alsaPreferredAudio,
         alsaSampleRate,
+        readAudio,
+        readMIDI,
+
         // end of engine controls
         addPresetRootDir = 60,
         removePresetRootDir,
@@ -309,7 +336,6 @@ namespace CONFIG // usage CONFIG::control::oscillatorSize
         bankRootCC = 65,
         bankCC = 67,
         enableProgramChange,
-        instChangeEnablesPart,
         extendedProgramChangeCC = 71,
         ignoreResetAllCCs,
         logIncomingCCs,
@@ -331,11 +357,13 @@ namespace BANK // usage BANK::control::
         deleteInstrument, // from bank
         selectFirstInstrumentToSwap,
         selectSecondInstrumentAndSwap,
+        lastSeenInBank,
 
         selectBank = 16, // in root, by ID or read ID + name
         renameBank, // or read just the name
-        createBank, // not yet - currently 'add' at top level
+        createBank,
         deleteBank, // not yet - currently 'remove' at top level
+        findBankSize,
         selectFirstBankToSwap,
         selectSecondBankAndSwap,
         importBank, // not yet (currently done in main)
@@ -343,8 +371,10 @@ namespace BANK // usage BANK::control::
 
         selectRoot = 32, // by ID - also reads the current one
         changeRootId, // change ID of current root
-        addNamedRoot, // not yet - currently add at top level
-        deselectRoot // not yet - currently remove at top level
+        addNamedRoot, // link or create if not already there
+        deselectRoot, // remove reference, but don't touch contents
+        installBanks,
+        refreshDefaults
     };
 }
 
@@ -405,7 +435,8 @@ namespace MIDILEARN // usage MIDILEARN::control::block
         loadList = 241,
         loadFromRecent,
         saveList = 245,
-        cancelLearn = 250
+        cancelLearn = 250,
+        learned
     };
 }
 
@@ -450,6 +481,8 @@ namespace MIDI // usage MIDI::control::noteOn
         pitchWheelInner = 128,
         channelPressureInner,
         keyPressureInner,
+        soloType,
+        soloCC,
 
         pitchWheel = 640,
         channelPressure,
@@ -461,41 +494,57 @@ namespace MIDI // usage MIDI::control::noteOn
         identNRPN = 0x8000,
         null
     };
+
+    enum SoloType : unsigned char {
+        Disabled = 0,
+        Row,
+        Column,
+        Loop,
+        TwoWay,
+        Channel
+    };
 }
 
 namespace SCALES // usage SCALES::control::refFrequency
 {
     enum control : unsigned char {
-        refFrequency = 0,
+        enableMicrotonal = 0,
+        refFrequency,
         refNote,
         invertScale,
         invertedScaleCenter,
         scaleShift,
-        enableMicrotonal = 8,
         enableKeyboardMap = 16,
         lowKey,
         middleKey,
         highKey,
         tuning = 32,
+        clearAll,
+        retune, // GUI only
+        // all the above directly alter the tuning. retune must be the last
+
         keyboardMap,
         importScl = 48,
         importKbm,
+        exportScl, // not yet
+        exportKbm, // not yet
         name = 64,
-        comment,
-        retune = 80, // GUI only
-        clearAll = 96
+        comment
     };
 }
 
 namespace MAIN // usage MAIN::control::volume
 {
     enum control : unsigned char {
-        volume = 0,
+        mono = 0,
+        volume,
         partNumber = 14,
         availableParts,
+        panLawType,
         detune = 32,
         keyShift = 35,
-        mono,
+        bpmFallback,
+        reseed = 40,
         soloType = 48,
         soloCC,
 
@@ -514,34 +563,50 @@ namespace MAIN // usage MAIN::control::volume
         saveNamedScale,
         loadNamedState = 92,
         saveNamedState,
+        readLastSeen,
         loadFileFromList,
+        defaultPart,
         exportPadSynthSamples,
-
-        masterReset = 96,
+        masterReset,
         masterResetAndMlearn,
-        openManualPDF = 100,
+        openManual = 100,
         startInstance = 104,
         stopInstance,
+        undo,
+        redo,
         stopSound = 128,
         readPartPeak = 200, // now does L/R
         readMainLRpeak,
         readMainLRrms
     };
+
+    enum panningType : unsigned char {
+        cut = 0,
+        normal,
+        boost
+    };
+
 }
 
 namespace PART // usage PART::control::volume
 {
     enum control : unsigned char {
-        volume = 0,
+        enable = 0,
+        enableAdd,
+        enableSub,
+        enablePad,
+        enableKitLine,
+        volume,
         velocitySense,
         panning,
-        velocityOffset = 4,
+        velocityOffset,
         midiChannel,
         keyMode,
+        channelATset,
+        keyATset,
         portamento,
-        enable,
         kitItemMute,
-        minNote = 16,
+        minNote,
         maxNote,
         minToLastKey,
         maxToLastKey,
@@ -561,12 +626,10 @@ namespace PART // usage PART::control::volume
         effectType,
         effectDestination,
         effectBypass,
-        defaultInstrument = 96,
-        padsynthParameters = 104,
         audioDestination = 120,
 
     // start of controllers
-        volumeRange = 128,
+        volumeRange = 128, // start marker (must be first)
         volumeEnable,
         panningWidth,
         modWheelDepth,
@@ -580,16 +643,17 @@ namespace PART // usage PART::control::volume
         filterQdepth,
         filterCutoffDepth,
         breathControlEnable,
-        resonanceCenterFrequencyDepth = 144,
+        resonanceCenterFrequencyDepth,
         resonanceBandwidthDepth,
-        portamentoTime = 160,
+        portamentoTime,
         portamentoTimeStretch,
         portamentoThreshold,
         portamentoThresholdType,
         enableProportionalPortamento,
         proportionalPortamentoRate,
         proportionalPortamentoDepth,
-        receivePortamento = 168,
+        receivePortamento, // 151
+        resetAllControllers, // end marker (must be last)
     // end of controllers
 
     // start of midi controls
@@ -601,6 +665,9 @@ namespace PART // usage PART::control::volume
         midiFilterQ,
         midiFilterCutoff,
         midiBandwidth,
+        midiFMamp,
+        midiResonanceCenter,
+        midiResonanceBandwidth,
     // end of midi controls
 
         instrumentCopyright = 220,
@@ -608,7 +675,6 @@ namespace PART // usage PART::control::volume
         instrumentName,
         instrumentType,
         defaultInstrumentCopyright, // this needs to be split into two for load/save
-        resetAllControllers, // this needs to bump up 1 to make space
         partBusy = 252 // internally generated - read only
     };
 
@@ -625,23 +691,35 @@ namespace PART // usage PART::control::volume
         padSynth,
 
     // addVoice and addMod must be consecutive
-        addVoice1 = 128,
-        addVoice2,
-        addVoice3,
-        addVoice4,
-        addVoice5,
-        addVoice6,
-        addVoice7,
-        addVoice8,
-        addMod1 = 192,
-        addMod2,
-        addMod3,
-        addMod4,
-        addMod5,
-        addMod6,
-        addMod7,
-        addMod8
+        addVoice1 = NUM_VOICES,
+        addMod1 = addVoice1 + NUM_VOICES,
+        addVoiceModEnd = addMod1 + NUM_VOICES
     };
+
+    namespace aftertouchType {
+        enum {  // all powers of 2 handled bit-wise
+            off = 0,
+            filterCutoff,
+            filterCutoffDown,
+            filterQ = 4,
+            filterQdown = 8,
+            pitchBend = 16,
+            pitchBendDown = 32,
+            volume = 64,
+            modulation = 128 // this MUST be the highest bit
+        };
+    }
+
+    namespace envelope
+    {
+        enum groupmode : int {
+            amplitudeLin = 1,
+            amplitudeLog,
+            frequency,
+            filter,
+            bandwidth
+        };
+    }
 }
 
 namespace ADDSYNTH // usage ADDSYNTH::control::volume
@@ -650,8 +728,8 @@ namespace ADDSYNTH // usage ADDSYNTH::control::volume
         volume = 0,
         velocitySense,
         panning,
-
-        enable = 8,
+        enableRandomPan,
+        randomWidth,
 
         detuneFrequency = 32,
         octave = 35,
@@ -673,12 +751,14 @@ namespace ADDSYNTH // usage ADDSYNTH::control::volume
 namespace ADDVOICE // usage ADDVOICE::control::volume
 {
     enum control : unsigned char {
-        volume = 0,
+        enableVoice = 0,
+        volume,
         velocitySense,
         panning,
-        invertPhase = 4,
-        enableAmplitudeEnvelope = 7,
-        enableVoice,
+        enableRandomPan,
+        randomWidth,
+        invertPhase,
+        enableAmplitudeEnvelope,
         enableAmplitudeLFO,
 
         modulatorType = 16, // Off, Morph, Ring, PM, FM, PWM
@@ -739,8 +819,8 @@ namespace SUBSYNTH // usage SUBSYNTH::control::volume
         volume = 0,
         velocitySense,
         panning,
-
-        enable = 8,
+        enableRandomPan,
+        randomWidth,
 
         bandwidth = 16,
         bandwidthScale,
@@ -777,12 +857,8 @@ namespace PADSYNTH // usage PADSYNTH::control::volume
         volume = 0,
         velocitySense,
         panning,
-
-        enable = 8,
-
-        bandwidth = 16,
-        bandwidthScale,
-        spectrumMode = 19, // Bandwidth, Discrete, Continuous
+        enableRandomPan,
+        randomWidth,
 
         detuneFrequency = 32,
         equalTemperVariation,
@@ -793,6 +869,11 @@ namespace PADSYNTH // usage PADSYNTH::control::volume
         pitchBendAdjustment,
         pitchBendOffset,
 
+        bandwidth,
+        bandwidthScale,       // Normal, Equal Hz, ¼ , ½ , ¾ , 1½ , Double, Inverse ½
+        spectrumMode,         // Bandwidth, Discrete, Continuous
+        xFadeUpdate,          // in millisec
+
         overtoneParameter1 = 48,
         overtoneParameter2,
         overtoneForceHarmonics,
@@ -815,6 +896,14 @@ namespace PADSYNTH // usage PADSYNTH::control::volume
         samplesPerOctave, // 0.5, 1, 2, 3, 4, 6, 12
         numberOfOctaves, // 1 - 8
         sampleSize, // 16k, 32k, 64k, 128k, 256k, 512k, 1M
+
+        rebuildTrigger = 90,
+        randWalkDetune,          // random walk spread, 0 off, 96 is factor 2
+        randWalkBandwidth,       // -> bandwidth
+        randWalkFilterFreq,      // -> centerFrequency
+        randWalkProfileWidth,    // -> baseWidth
+        randWalkProfileStretch,  // -> modulatorStretch
+
         applyChanges = 104,
         stereo = 112,
 
@@ -868,15 +957,36 @@ namespace OSCILLATOR // usage OSCILLATOR::control::phaseRandomness
         clearHarmonics = 96,
         convertToSine
     };
+    enum wave : unsigned char {
+        sine = 0,
+        triangle,
+        pulse,
+        saw,
+        power,
+        gauss,
+        diode,
+        absSine,
+        pulseSine,
+        stretchSine,
+        chirp,
+        absStretchSine,
+        chebyshev,
+        square,
+        spike,
+        circle,
+        hyperSec,
+        user = 127
+    };
 }
 
 namespace RESONANCE // usage RESONANCE::control::maxDb
 {
     enum control : unsigned char {
-        maxDb = 0,
+        enableResonance = 0,
+        maxDb,
         centerFrequency,
         octaves,
-        enableResonance = 8,
+
         randomType = 10, // coarse, medium, fine
         interpolatePeaks = 20, // smooth, linear
         protectFundamental,
@@ -896,6 +1006,7 @@ namespace LFOINSERT // usage LFOINSERT::control::speed
         amplitudeRandomness,
         type, // Sine, Tri, Sqr, R.up, R.dn, E1dn, E2dn
         continuous,
+        bpm,
         frequencyRandomness,
         stretch
     };
@@ -954,26 +1065,15 @@ namespace ENVELOPEINSERT // usage ENVELOPEINSERT::control::attackLevel
     };
 }
 
-namespace EFFECT // usage EFFECT::type::none
+namespace EFFECT // usage EFFECT::control::level
 {
-    enum type : unsigned char {
-        none = 128, // must be higher than normal kits
-        reverb,
-        echo,
-        chorus,
-        phaser,
-        alienWah,
-        distortion,
-        eq,
-        dynFilter,
-        count // this must be the last item!
-    };
-
     enum control : unsigned char {
         level = 0, // volume, wet/dry, gain for EQ
         panning, // band for EQ
         frequency, // time reverb, delay echo, L/R-mix dist, Not EQ
         preset = 16, // not EQ
+        bpm,
+        bpmStart,
         changed = 129 // not EQ
     };
 
@@ -988,12 +1088,12 @@ namespace EFFECT // usage EFFECT::type::none
     };
 }
 
+/*
+ * it is ESSENTIAL that the size is a power of 2
+ */
 union CommandBlock{
     struct{
-        union{
-            float F;
-            int32_t I;
-        } value;
+        float value;
         unsigned char type;
         unsigned char source;
         unsigned char control;
@@ -1009,9 +1109,5 @@ union CommandBlock{
     } data;
     char bytes [sizeof(data)];
 };
-/*
- * it is ESSENTIAL that this is a power of 2
- */
-const size_t commandBlockSize = sizeof(CommandBlock);
 
 #endif
